#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
import random
from math import acos, asin, atan, pi, sin, cos, tan
from pyromaths.ex import LegacyExercise

from pyromaths.outils.Arithmetique import valeur_alea, pgcd
from pyromaths.outils.Geometrie import choix_points
from pyromaths.outils.decimaux import decimaux
from . import fractions
from decimal import Decimal


# ---------------------------------------------------------------------
# -                    THÉORÈME DE THALÈS                             -
# ---------------------------------------------------------------------
def cotes_sommets(noms):  # renvoie les noms des 3 cotes du triangle en finissant par l'hypotenuse
    return (noms[1] + noms[2], noms[0] + noms[2], noms[0] + noms[1])


def nom_triangle(noms):  # renvoie le nom du triangle dans un ordre aleatoire
    a = random.randrange(3)
    b = (random.randrange(2) + 1 + a) % 3
    c = (3 - a) - b
    return '%s%s%s' % (noms[a], noms[b], noms[c])


def valeurs_thales(valeurmax, typeexo):
    """typeexo est égal à 1 pour la version triangle, -1 pour la version papillon"""
    liste = [0, 0, 0, 0, 0, 0, 0, 0]
    while liste == [0, 0, 0, 0, 0, 0, 0, 0]:
        for i in range(3):
            liste[i] = random.randrange(2)
    a = random.randrange(liste.count(1))
    for i in range(3):
        if liste[i]:
            if not a:
                liste[i + 3] = 1
            a = a - 1
        else:
            liste[i + 3] = 1  # on doit connaitre le numeratuer ou le denominateur
    for i in range(2):  # AB et AE  ou  AB et BE  ou  AE et EB
        if liste[i] and liste[i + 3]:  # i est le rapport complet. On choisit une des 3 formes ci-dessus
            a = random.randrange(2)
            liste[i + 6] = 1
            liste[i + 3 * a] = 0
            rapport = [i + 3 * ((a + 1) % 3), i + 3 * ((a + 2) % 3)]
            rapport.sort()
    if liste[2] and liste[5]:
        rapport = [2, 5]
    valeurs = [0, 0, 0, 0, 0, 0, 0, 0]
    for i in range(3):
        if liste[i]:
            valeurs[i] = random.randrange(15, valeurmax) / 10
        if liste[i + 3] and liste[i]:
            valeurs[i + 3] = random.randrange(5, valeurs[i] * 10 - 9) / 10
        elif liste[i + 3]:
            valeurs[i + 3] = random.randrange(5, valeurmax) / 10
    if liste[6]:
        valeurs[6] = random.randrange(5, valeurmax) / 10
    if liste[7]:
        valeurs[7] = random.randrange(5, valeurmax) / 10

    valeurs.append((rapport, typeexo))
    if test_valeurs_thales(valeurs, rapport, typeexo):
        return valeurs
    else:
        return 0


def test_valeurs_thales(valeurs, rapport, type_thales):
    v = [valeurs[i] for i in range(8)]
    if rapport[0] // 3 == 0 and rapport[1] // 3 == 2:  # On donne AB et EB
        v[rapport[0] + 3] = (v[rapport[0]] - v[rapport[1]]) * \
                            type_thales
    elif rapport[0] // 3 == 1 and rapport[1] // 3 == 2:

        # On donne AE et EB

        v[rapport[0] - 3] = v[rapport[0]] * type_thales + v[rapport[1]]
    if v[rapport[0] % 3]:  # rapport est AE/AB
        rapp = v[rapport[0] % 3 + 3] / v[rapport[0] % 3]
    else:
        rapp = 0
    for i in range(3):
        if not v[i] and rapp:
            v[i] = v[i + 3] / rapp
        elif not v[i + 3]:
            v[i + 3] = v[i] * rapp
    if inegalite_triangulaire(v[0:3]) and inegalite_triangulaire(v[3:6]) and .3 < rapp < .7:
        return v
    else:
        return 0


def inegalite_triangulaire(a):  # renvoie 1 si c'est un triangle, 0 sinon
    vrai = 0
    coef = 1.2  # evite les triangles trop ecrases
    if a[0] > a[1] and a[0] > a[2]:
        if a[1] + a[2] > coef * a[0]:
            vrai = 1
    elif a[1] > a[0] and a[1] > a[2]:
        if a[0] + a[2] > coef * a[1]:
            vrai = 1
    elif a[2] > a[0] and a[2] > a[1]:
        if a[0] + a[1] > coef * a[2]:
            vrai = 1
    return vrai


def thales(exo, cor):
    typeexo = [-1, 1]
    random.shuffle(typeexo)
    for i in range(2):
        arrondi = random.randrange(1, 4)
        text_arrondi = ['dix', 'cent', 'mill'][arrondi - 1] + u'ième'
        noms = choix_points(5)  # les noms des sommets
        while True:
            valeurs = valeurs_thales(70, typeexo[i])  # les longueurs en mm
            if valeurs:
                break
        exo.append(r"\figureadroite{%")
        cor.append(r"\figureadroite{%")
        exo.append(tex_fig_thales(noms, valeurs))
        cor.append(tex_fig_thales(noms, valeurs))
        exo.append("}{%")
        cor.append("}{%")
        exo.append(tex_enonce_thales(noms, valeurs, text_arrondi))
        cor.append(tex_enonce_thales(noms, valeurs, text_arrondi))
        exo.append(r"}\par")
        cor.append(r"}\par")
        cor.append(tex_resolution_thales0(noms))
        cor.append(tex_resolution_thales1(noms, valeurs))
        cor.append(tex_resolution_thales2(noms, valeurs))
        cor.append(tex_resolution_thales3(noms, valeurs, arrondi))


def long_val(noms,
             valeurs):  # renvoie un tuple contenant les noms des segments et leur longueur puis les noms des longueurs a calculer
    liste = []
    for i in range(8):
        if valeurs[i]:
            liste.append(creer_noms(noms, i))
            liste.append(decimaux(valeurs[i]))
    for i in range(6):
        if not valeurs[i] and valeurs[8][0][0] % 3 != i % 3:
            liste.append(creer_noms(noms, i))
    return liste


def lAB(a):  # renvoie AB
    return str(a[0]) + str(a[1])

def creer_noms(noms, i):
    if i == 0:
        return str(noms[0]) + str(noms[1])
    elif i == 1:
        return str(noms[0]) + str(noms[2])
    elif i == 2:
        return str(noms[1]) + str(noms[2])
    elif i == 3:
        return str(noms[0]) + str(noms[3])
    elif i == 4:
        return str(noms[0]) + str(noms[4])
    elif i == 5:
        return str(noms[3]) + str(noms[4])
    elif i == 6:
        return str(noms[3]) + str(noms[1])
    elif i == 7:
        return str(noms[4]) + str(noms[2])


def tex_enonce_thales(noms, valeurs, text_arrondi):
    texte = \
        u'Sur la figure ci-contre, les droites $(%s)\\text{ et }(%s)$ sont parallèles.\\par\n' % \
        (lAB(noms[1:3]), lAB(noms[3:5]))
    liste = long_val(noms, valeurs)
    texte += \
        'On donne $%s=\\unit[%s]{cm}$ \\quad $%s=\\unit[%s]{cm}$ \\quad $%s=\\unit[%s]{cm}$ \\quad $%s~=~\\unit[%s]{cm}$.\\par\n' % \
        tuple(liste[0:8])
    texte += 'Calculer $%s$ et $%s$, ' % tuple(liste[8:10])
    texte += u'arrondies au %s.' % text_arrondi
    return texte


def tex_resolution_thales0(n):
    return u"""Les points $%s$, $%s$, $%s$ et $%s$, $%s$, $%s$ sont alignés et les droites $(%s)$ et $(%s)$ sont parallèles.\\par
D'après le \\textbf{théorème de Thalès} : $\\quad\\mathbf{\\cfrac{%s}{%s}=\\cfrac{%s}{%s}=\\cfrac{%s}{%s}}$""" % \
           (
               n[0],
               n[3],
               n[1],
               n[0],
               n[4],
               n[2],
               n[1] + n[2],
               n[3] + n[4],
               creer_noms(n, 0),
               creer_noms(n, 3),
               creer_noms(n, 1),
               creer_noms(n, 4),
               creer_noms(n, 2),
               creer_noms(n, 5),
           )


def tex_resolution_thales1(n, v):
    r = v[8][0][0] % 3  # grand rapport
    if v[8][1] == 1:
        sgn = '+'
    else:
        sgn = '-'
    if v[r] and v[r + 3]:  # on connait les deux rapports
        donnees = 0
    elif v[r + 3]:

        # on connait  le petit rapport, mais pas le grand

        v[r] = v[r + 6] + v[r + 3] * v[8][1]
        donnees = (creer_noms(n, r), creer_noms(n, r + 6), sgn,
                   creer_noms(n, r + 3), decimaux(v[r]))
    else:
        v[r + 3] = (v[r] - v[r + 6]) * v[8][1]
        if sgn == '+':
            donnees = (creer_noms(n, r + 3), creer_noms(n, r), '-',
                       creer_noms(n, r + 6), decimaux(v[r + 3]))
        else:
            donnees = (creer_noms(n, r + 3), creer_noms(n, r + 6), '-',
                       creer_noms(n, r), decimaux(v[r + 3]))
    if donnees:
        return u'\\par\\medskip\nDe plus $%s=%s%s%s=\\unit[%s]{cm}$, d\'où' % \
               donnees
    else:
        return u'\\quad d\'où'


def tex_resolution_thales2(n, v):
    donnees = []
    for i in range(3):
        if v[i]:
            donnees.append(decimaux(v[i]))
        else:
            donnees.append(creer_noms(n, i))
        if v[i + 3]:
            donnees.append(decimaux(v[i + 3]))
        else:
            donnees.append(creer_noms(n, i + 3))
    return '$\\cfrac{%s}{%s}=\\cfrac{%s}{%s}=\\cfrac{%s}{%s}$\\par' % \
           tuple(donnees)


def nom_ou_valeur(n, v, i):
    if v[i]:
        return decimaux(v[i])
    else:
        return creer_noms(n, i)


def valeur_exacte(a, approx=3, unit=1):
    nb = decimaux(a)
    if unit:
        if nb.count(',') and (len(nb) - nb.find(',')) - 1 > approx:
            return '\\simeq\\unit[' + decimaux(round(a, approx)) + ']{cm}'
        else:
            return '=\\unit[' + decimaux(a) + ']{cm}'
    else:
        if nb.count(',') and (len(nb) - nb.find(',')) - 1 > approx:
            return '\\simeq' + decimaux(round(a, approx))
        else:
            return '=' + decimaux(a)


def tex_resolution_thales3(n, v, arrondi):
    r = v[8][0][0] % 3  # grand rapport
    donnees = []
    for i in range(3):
        if i != r:
            donnees.extend([nom_ou_valeur(n, v, r), nom_ou_valeur(n, v,
                                                                  r + 3), nom_ou_valeur(n, v, i), nom_ou_valeur(n,
                                                                                                                v,
                                                                                                                i + 3)])
            if v[i]:  # on cherche i+3
                donnees.extend([creer_noms(n, i + 3), decimaux(v[i]), decimaux(v[r + 3]), decimaux(v[r]),
                                valeur_exacte(v[i] * v[r + 3] / v[r], approx=arrondi)])
            else:
                donnees.extend([creer_noms(n, i), decimaux(v[i + 3]), decimaux(v[r]), decimaux(v[r + 3]),
                                valeur_exacte(v[r] * v[i + 3] / v[r + 3], approx=arrondi)])
    texte = \
        '$\\cfrac{%s}{%s}=\\cfrac{%s}{%s}\\quad$ donc $\\boxed{%s=\\cfrac{%s\\times %s}{%s}%s}$\\hfill' % \
        tuple(donnees[0:9])
    texte = texte + \
            '$\\cfrac{%s}{%s}=\\cfrac{%s}{%s}\\quad$ donc $\\boxed{%s=\\cfrac{%s\\times %s}{%s}%s}$\\par\n' % \
            tuple(donnees[9:18])
    return texte


def fig_thales(noms, valeurs):
    v = test_valeurs_thales(valeurs[0:8], valeurs[8][0], valeurs[8][1])
    type_thales = valeurs[8][1]
    angle = Decimal(int(100 * acos((v[0] ** 2 + v[1] ** 2 - v[2] ** 2) / ((2 * v[0]) * v[1])) * 180 / pi)) / 100
    v = [int(v[i] * 100) / 100 for i in range(8)]
    mini_x = int(100 * min(0, v[1] * cos(float(angle) * pi / 180), v[3] * type_thales,
                           (v[4] * cos(float(angle) * pi / 180)) * type_thales)) / 100 - 1.5
    mini_y = int(100 * min(0, (v[4] * sin(float(angle) * pi / 180)) * type_thales)) / 100 - 1.5
    maxi_x = int(100 * max(v[0], v[1] * cos(float(angle) * pi / 180))) / 100 + 1.5
    maxi_y = int((100.0 * v[1]) * sin(float(angle) * pi / 180)) / 100 + .5
    echelle = int(400 / max(abs(mini_x) + maxi_x, abs(mini_y) + maxi_y)) / 100
    if type_thales == 1:
        return (echelle, mini_x, mini_y, maxi_x, maxi_y, 225, angle + 45, noms[0], noms[1], noms[2], v[0], v[1], angle,
                - 45, angle + 90, noms[3], noms[4], v[3], v[4], angle,)
    else:
        return (echelle, mini_x, mini_y, maxi_x, maxi_y, 135, angle + 45, noms[0], noms[1], noms[2], v[0], v[1], angle,
                135, angle + 180, noms[3], noms[4], - v[3], - v[4], angle,)


def tex_fig_thales(noms, valeurs):
    donnees = fig_thales(noms, valeurs)
    enonce = \
        '''\\psset{PointSymbol=none,unit=%s}
\\begin{pspicture}(%s,%s)(%s,%s)
\\SpecialCoor
\\pstTriangle[PosAngleA=%s, PosAngleB=-45, PosAngleC=%s, PointNameA=%s, PointNameB=%s, PointNameC=%s](0,0){a}(%s,0){b}(%s;%s){c}
\\pstTriangle[PosAngleB=%s, PosAngleC=%s, PointSymbolA=none, PointNameA=none, PointNameB=%s, PointNameC=%s](0,0){a}(%s,0){b}(%s;%s){c}
\\end{pspicture}''' % donnees
    return enonce


def _tex_thales():
    exo = ['\\exercice']
    cor = ['\\exercice*']
    thales(exo, cor)
    return exo, cor


class tex_thales(LegacyExercise):
    """Théorème de Thalès"""

    tags = ["Troisième"]
    function = _tex_thales


#
# ------------------- RECIPROQUE DU THEOREME DE THALES -------------------
#


def valeurs_reciproque_thales():
    while True:
        (a, b, c, d) = (random.randrange(2, 50), random.randrange(2, 50),
                        random.randrange(2, 20), random.randrange(2, 20))
        p1 = pgcd(a, b)
        (a, b) = (a // p1, b // p1)
        if c < d:
            (c, d) = (d, c)
        if a != b and int(c / d) != c / d and 10 < a * c < 200 and 10 < a * d < 200 and 10 < b * c < 200 and \
                10 < b * d < 200 and .3 < d / c < .7:
            break
    t = valeur_alea(-1, 1)  # -1 si papillon, 1 si triangle
    r = random.randrange(5)
    while r == 2:
        r = random.randrange(5)
    angle = random.randrange(15, 105)
    valeurs = (
        (a * c) / 10,
        (b * c) / 10,
        0,
        (a * d) / 10,
        (b * d) / 10,
        0,
        (a * c - (t * a) * d) / 10,
        (b * c - (t * b) * d) / 10,
        angle,
        t,
        r,
    )
    return valeurs


def fig_rec_thales(noms, v):
    type_thales = v[9]
    angle = v[8]
    mini_x = int(100 * min(0, v[1] * cos(angle * pi / 180), v[3] * type_thales,
                           (v[4] * cos(angle * pi / 180)) * type_thales)) / 100 - 1.5
    mini_y = int(100 * min(0, (v[4] * sin(angle * pi / 180)) * type_thales)) / 100 - 1.5
    maxi_x = int(100 * max(v[0], v[1] * cos(angle * pi / 180))) / 100 + 1.5
    maxi_y = int(100 * v[1] * sin(angle * pi / 180)) / 100 + .5
    echelle = int(400 / max(abs(mini_x) + maxi_x, abs(mini_y) + maxi_y)) / 100
    if type_thales == 1:
        return (echelle, mini_x, mini_y, maxi_x, maxi_y, 225, angle + 45, noms[0], noms[1], noms[2], v[0], v[1], angle,
                - 45, angle + 90, noms[3], noms[4], v[3], v[4], angle,)
    else:
        return (echelle, mini_x, mini_y, maxi_x, maxi_y, 135, angle + 45, noms[0], noms[1], noms[2], v[0], v[1], angle,
                135, angle + 180, noms[3], noms[4], - v[3], - v[4], angle,)


def tex_fig_rec_thales(noms, valeurs):
    donnees = fig_rec_thales(noms, valeurs)
    enonce = \
        '''{\\begin{wrapfigure}{r}{4cm}
\\psset{PointSymbol=none,unit=%s}
\\begin{pspicture}(%s,%s)(%s,%s)
\\SpecialCoor
\\pstTriangle[PosAngleA=%s,PosAngleB=-45,PosAngleC=%s,PointNameA=%s,PointNameB=%s,PointNameC=%s](0,0){a}(%s,0){b}(%s;%s){c}
\\pstTriangle[PosAngleB=%s,PosAngleC=%s,PointSymbolA=none,PointNameA=none,PointNameB=%s,PointNameC=%s](0,0){a}(%s,0){b}(%s;%s){c}
\\end{pspicture}
\\end{wrapfigure}\\par
''' % \
        donnees
    return enonce


def rec_thales(exo, cor):
    noms = choix_points(5)  # les noms des sommets
    valeurs = valeurs_reciproque_thales()
    exo.append(tex_fig_rec_thales(noms, valeurs))
    exo.append(tex_enonce_rec_thales(noms, valeurs) + '\\vspace{2cm}}')  # le dernier '}' ferme le bloc exercice
    cor.append(tex_fig_rec_thales(noms, valeurs))
    cor.append(tex_enonce_rec_thales(noms, valeurs) +
               "\\par\\dotfill{}\\\\}\n")
    cor.append(tex_resolution_rec_thales0(noms, valeurs))
    cor.append(tex_resolution_rec_thales1(noms, valeurs))

    #  cor.append(tex_resolution_rec_thales2(noms, valeurs))
    # cor.append(tex_resolution_rec_thales3(noms, valeurs))


def enonce_rec_thales(n, v):
    (r, l) = (v[10], [])
    for i in range(5):
        if i != 2:
            if i == r:
                l.extend([creer_noms(n, 6 + i % 3), decimaux(v[6 + i % 3])])
            else:
                l.extend([creer_noms(n, i), decimaux(v[i])])
    l1 = [i for i in range(4)]
    l2 = []
    for i in range(4):
        a = l1.pop(random.randrange(4 - i))
        l2.extend([l[2 * a], l[2 * a + 1]])
    l2.extend([creer_noms(n, 2), creer_noms(n, 5)])
    return tuple(l2)


def tex_enonce_rec_thales(n, v):
    d = enonce_rec_thales(n, v)
    texte = \
        u'''Sur la figure ci-contre, on donne $%s=\\unit[%s]{cm}$, $%s=\\unit[%s]{cm}$, $%s=\\unit[%s]{cm}$ et $%s=\\unit[%s]{cm}$.\\par
Démontrer que les droites $(%s)$ et $(%s)$ sont parallèles.
''' % \
        d
    return texte


def resolution_rec_thales0(n, v):
    t = v[9]
    r = v[10]
    if t > 0:
        d = [n[0], n[3], n[1], n[0], n[4], n[2], creer_noms(n, r)]
    else:
        d = [n[3], n[0], n[1], n[4], n[0], n[2], creer_noms(n, r)]
    if r < 2:
        if t > 0:
            d.extend([creer_noms(n, r + 6), '+', creer_noms(n, r + 3),
                      decimaux(v[r])])
        else:
            d.extend([creer_noms(n, r + 6), '-', creer_noms(n, r + 3),
                      decimaux(v[r])])
    else:
        if t > 0:
            d.extend([creer_noms(n, r - 3), '-', creer_noms(n, r + 3),
                      decimaux(v[r])])
        else:
            d.extend([creer_noms(n, r + 3), '-', creer_noms(n, r - 3),
                      decimaux(v[r])])
    return tuple(d)


def tex_resolution_rec_thales0(n, v):
    return u"""Les points $%s$, $%s$, $%s$ et $%s$, $%s$, $%s$ sont alignés dans le même ordre.\\par
De plus $%s=%s%s%s=\\unit[%s]{cm}$.\\par
""" % \
           resolution_rec_thales0(n, v)


def resolution_rec_thales1(n, v):
    (d, t) = ([], '')
    for i in range(2):
        d.extend([creer_noms(n, i), creer_noms(n, i + 3), decimaux(v[i]),
                  decimaux(v[i + 3])])
        if valeur_exacte(v[i] / v[i + 3], approx=5).count('='):
            d.append(valeur_exacte(v[i] / v[i + 3], approx=5, unit=0))
        else:
            if v[i] != int(v[i]) or v[i + 3] != int(v[i + 3]):
                p = pgcd(int(v[i] * 10), int(v[i + 3] * 10))
                if p == 1:
                    t = '=\\cfrac{%s}{%s}' % (decimaux(v[i] * 10), decimaux(v[i +
                                                                          3] * 10))
                else:
                    t = '=\\cfrac{%s_{\\div%s}}{%s_{\\div%s}}' % (decimaux(v[i] *
                                                                         10), p, decimaux(v[i + 3] * 10), p)
            if fractions.simplifie((int(v[i] * 10), int(v[i + 3] * 10))):
                d.append(t + '=' + fractions.tex_frac(fractions.simplifie((int(v[i] *
                                                                               10), int(v[i + 3] * 10)))))
            else:
                d.append(t + '=' + fractions.tex_frac((int(v[i] * 10),
                                                       int(v[i + 3] * 10))))
    d.extend([creer_noms(n, 0), creer_noms(n, 3), creer_noms(n, 1),
              creer_noms(n, 4), creer_noms(n, 2), creer_noms(n, 5)])
    return tuple(d)


def tex_resolution_rec_thales1(n, v):
    d = resolution_rec_thales1(n, v)
    return u"""$\\left.
\\renewcommand{\\arraystretch}{2}
\\begin{array}{l}
\\bullet\\cfrac{%s}{%s}=\\cfrac{%s}{%s}%s\\\\\n    \\bullet\\cfrac{%s}{%s}=\\cfrac{%s}{%s}%s
\\end{array}
\\right\\rbrace$
Donc $\\cfrac{%s}{%s}=\\cfrac{%s}{%s}$\\,.\\par
D'après la \\textbf{réciproque du théorème de Thalès}, \\fbox{les droites $(%s)$ et $(%s)$ sont parallèles.}
""" % \
           d


def _tex_reciproque_thales():
    exo = ['\\exercice']
    cor = ['\\exercice*']
    rec_thales(exo, cor)
    return exo, cor

class tex_reciproque_thales(LegacyExercise):
    """Réciproque du théorème de Thalès"""

    tags = ["Troisième"]
    function = _tex_reciproque_thales

#
# ------------------- TRIGONOMETRIE -------------------
#

def trigo_init(exo, cor):
    s = choix_points(6)
    n1 = cotes_sommets(s[0:3])
    n2 = cotes_sommets(s[3:6])
    v = valeurs_trigo()
    enonce_trigo(exo, cor, ((s[0:3], n1, v[0]), (s[3:6], n2, v[1])))


def enonce_trigo(exo, cor, v):
    (l, lt) = ([], [])
    arrondi = random.randrange(1, 4)
    text_arrondi = ['dix', 'cent', 'mill'][arrondi - 1] + u'ième'
    for j in range(2):
        f = (('\\sin', 1, 0), ('\\cos', 2, 0), ('\\tan', 1, 2))[v[j][2][0]]
        for i in range(2):
            l.append(v[j][1][f[i + 1]])
            l.append(v[j][2][i + 1])
        l.append(tex_angle(v[j][0], 1))
        l.append(v[j][2][3])
    for j in range(2):
        tmp = []
        for i in range(3):
            if len(l[2 * i + 6 * j]) < 3:
                if l[2 * i + 6 * j + 1]:
                    lt.append('$%s=\\unit[%s]{cm}$' % (l[2 * i + 6 * j],
                                                       decimaux(l[2 * i + 6 * j + 1])))
                else:
                    tmp = 'la longueur $%s$' % l[2 * i + 6 * j]
            elif l[2 * i + 6 * j + 1]:
                lt.append('$%s=%s\\degres$' % (l[2 * i + 6 * j], l[2 * i +
                                                                   6 * j + 1]))
            else:
                lt.append('la mesure de l\'angle $%s$' % l[2 * i + 6 * j])
        if tmp:
            lt.append(tmp)
    exo.append('\\begin{multicols}{2}')
    exo.append('\\begin{enumerate}')
    cor.append('\\begin{multicols}{2}')
    cor.append('\\begin{enumerate}')
    tr = nom_triangle(v[0][0])
    exo.append('\\item $%s$ est un triangle rectangle en $%s$ tel que :\\par ' %
               (tr, v[0][0][0]))
    exo.append('%s et %s.\\par\nCalculer %s, arrondie au %s.\\par' % tuple(lt[0:3] + [text_arrondi]))
    cor.append('\\item $%s$ est un triangle rectangle en $%s$ tel que :\\par ' %
               (tr, v[0][0][0]))
    cor.append('%s et %s.\\par\nCalculer %s, arrondie au %s.\\par' % tuple(lt[0:3] + [text_arrondi]))
    cor.append("\\dotfill{}\\par\\vspace{2ex}")
    cor.append('Dans le triangle $%s$ rectangle en $%s$,' % (tr, v[0][0][0]))  # résolution
    v2 = (v[0][1], v[0][2])
    l2 = l[0:6]
    resolution_trigo(cor, v2, l2, arrondi)
    tr = nom_triangle(v[1][0])
    exo.append('\\columnbreak')
    cor.append('\\columnbreak')
    arrondi = random.randrange(1, 4)
    text_arrondi = ['dix', 'cent', 'mill'][arrondi - 1] + u'ième'
    exo.append('\\item $%s$ est un triangle rectangle en $%s$ tel que :\\par' %
               (tr, v[1][0][0]))
    exo.append('%s et %s.\\par\nCalculer %s, arrondie au %s.\\par' % tuple(lt[3:6] + [text_arrondi]))
    cor.append('\\item $%s$ est un triangle rectangle en $%s$ tel que :\\par' %
               (tr, v[1][0][0]))
    cor.append('%s et %s.\\par\nCalculer %s, arrondie au %s.\\par' % tuple(lt[3:6] + [text_arrondi]))
    cor.append("\\dotfill{}\\par\\vspace{2ex}")
    cor.append('Dans le triangle $%s$ rectangle en $%s$,' % (tr, v[1][0][0]))  # résolution
    v2 = (v[1][1], v[1][2])
    l2 = l[6:12]
    resolution_trigo(cor, v2, l2, arrondi)
    exo.append('\\end{enumerate}')
    exo.append('\\end{multicols}')
    cor.append('\\end{enumerate}')
    cor.append('\\end{multicols}')


def resolution_trigo(cor, v2, l2, arrondi):
    f = (('\\sin', 1, 0), ('\\cos', 2, 0), ('\\tan', 1, 2))[v2[1][0]]
    cor.append(u'\\[%s%s=\\cfrac{%s}{%s}' % (f[0], l2[4], v2[0][f[1]], v2[0][f[2]]) + '\\] ')
    if not v2[1][3]:
        cor.append(u'\\[ %s%s=\\cfrac{%s}{%s}' % (f[0], l2[4], decimaux(v2[1][1]),
                                                  decimaux(v2[1][2])) + '\\] ')
        if f[0] == '\\sin':
            r = asin(v2[1][1] / v2[1][2]) * 180 / pi
        elif f[0] == '\\cos':
            r = acos(v2[1][1] / v2[1][2]) * 180 / pi
        else:
            r = atan(v2[1][1] / v2[1][2]) * 180 / pi
        cor.append(r'\[ \boxed{%s=%s^{-1}\left(\cfrac{%s}{%s}\right) %s\degres} \]' %
                   (l2[4], f[0], decimaux(v2[1][1]), v2[1][2], valeur_exacte(r, approx=arrondi, unit=0)))
    elif not v2[1][1]:
        cor.append(u'\\[ %s%s=\\cfrac{%s}{%s}' % (f[0], v2[1][3], v2[0][f[1]],
                                                  decimaux(v2[1][2])) + '\\] ')
        if f[0] == '\\sin':
            r = sin((v2[1][3] * pi) / 180)
        elif f[0] == '\\cos':
            r = cos((v2[1][3] * pi) / 180)
        else:
            r = tan((v2[1][3] * pi) / 180)
        r = r * v2[1][2]
        cor.append(r'\[ \boxed{%s=%s%s\times %s %s } \]' % (v2[0][f[1]],
                                                            f[0], v2[1][3], decimaux(v2[1][2]),
                                                            valeur_exacte(r, approx=arrondi)))
    else:
        cor.append(u'\\[ %s%s=\\cfrac{%s}{%s}' % (f[0], v2[1][3], decimaux(v2[1][1]),
                                                  v2[0][f[2]]) + '\\] ')
        if f[0] == '\\sin':
            r = sin(v2[1][3] * pi / 180)
        elif f[0] == '\\cos':
            r = cos(v2[1][3] * pi / 180)
        else:
            r = tan(v2[1][3] * pi / 180)
        r = v2[1][1] / r
        cor.append(r'\[ \boxed{%s=\cfrac{%s}{%s%s} %s} \]' % (v2[0][f[2]],
                                                              decimaux(v2[1][1]), f[0], v2[1][3],
                                                              valeur_exacte(r, approx=arrondi)))


def tex_angle(s,
              n):  # renvoie \\widehat{ABC} où s est la liste des 3 sommets du triangle et n est le rang du sommet de l'angle dans cette liste
    return '\\widehat{%s%s%s}' % (s[(n + 2) % 3], s[n], s[(n + 1) % 3])


def valeurs_trigo():
    l = [random.randrange(10, 121) / 10 for i in range(3)]
    l.sort()
    l.append(random.randrange(15, 76))
    trigo = random.randrange(3)
    if random.randrange(2):  # on choisit en 1er une longueur et un angle
        if random.randrange(2):  # on connait la première des deux longueurs
            v = (trigo, l[0], 0, l[3])
        else:
            v = (trigo, 0, l[0], l[3])
        trigo = (trigo + 1 + random.randrange(2)) % 3
        v = (v, (trigo, l[1], l[2], 0))
    else:

        # on choisit en 1er deux longueurs

        v = (trigo, l[1], l[2], 0)
        trigo = (trigo + 1 + random.randrange(2)) % 3
        if random.randrange(2):  # on connait la première des deux longueurs
            v = (v, (trigo, l[0], 0, l[3]))
        else:
            v = (v, (trigo, 0, l[0], l[3]))
    return v

def _tex_trigo():
    exo = ['\\exercice']
    cor = ['\\exercice*']
    trigo_init(exo, cor)
    return exo, cor

class tex_trigo(LegacyExercise):
    """Trigonométrie"""

    tags = ["Troisième"]
    function = _tex_trigo



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import random
import math
from pyromaths.ex import LegacyExercise

#===============================================================================
# Fractions partage
#===============================================================================


def dimensions_rectangle():
    taille_max = 8
    while True:
        (l, h) = (random.randrange(4, taille_max), random.randrange(4,
                  taille_max))
        div_t = diviseurs(l * h)
        if len(div_t) > 3:
            break
    if l < h:
        (l, h) = (h, l)
    return (l, h)


def numerateur_denominateur(l, h, cas):
    """

    @param l: largeur
    @param h: longueur
    @param cas:
        - nid: numerateur < denominateur
        - un: numerateur = denominateur
        - nsd: numerateur > denominateur
    @type cas: string
    """

    ldiv = diviseurs(l * h)
    if cas == "un":
        d = random.randrange(3, l * h)
    else:
        while True:
            d = ldiv[random.randrange(len(ldiv) - 1)]
            if d > 2:
                break
    if cas == "un":
        n = d
    elif cas == "nid":
        n = random.randrange(1, d)
    else:
        n = random.randrange(d + 1, d * 2)
    return (n, d)


def trace_rectangle(exo, cor, l, h, cas):
    exo.append("\\psgrid[gridcolor=Olive,subgriddiv=0,gridlabels=0pt]")
    cor.append("\\psgrid[gridcolor=Olive,subgriddiv=0,gridlabels=0pt]")
    exo.append("\\psframe[linewidth=1.5\\pslinewidth,linecolor=Maroon](0,0)(%s,%s)" % (l, h))
    cor.append("\\psframe[linewidth=1.5\\pslinewidth,linecolor=Maroon](0,0)(%s,%s)" % (l, h))
    if cas == "nsd":
        cor.append("\\psframe[linewidth=1.5\\pslinewidth,linecolor=Maroon](%s,0)(%s,%s)" % 
                (l + 1, 2 * l + 1, h))

def fractions_partage_corrige(l, h, n, d):
    div_l = diviseurs(l)
    div_h = diviseurs(h)
    div_d = diviseurs(d)
    (lc, hc) = (l, h)
    if n == d:
        (lc, hc) = (l, h)
    elif div_l.count(d):
        lc = l // d
    elif div_h.count(d):
        hc = h // d
    else:
        for i in range(len(div_d) - 1):
            if div_l.count(div_d[i + 1]) and div_h.count(d // div_d[i + 1]):
                (lc, hc) = (l // div_d[i + 1], (h * div_d[i + 1]) // d)
                break
    return (lc, hc)


def trace_partage(cor, l, h, lc, hc, cas):
    if lc < l:
        cor.append("\\multips(%s,0)(%s,0){%s}{\\psline[linecolor=Maroon](0,0)(0,%s)}" % 
                 (lc, lc, l // lc - 1, h))
        if cas == "nsd":
            cor.append("\\rput(%s,0){\\multips(%s,0)(%s,0){%s}{\\psline[linecolor=Maroon](0,0)(0,%s)}}" % 
                     (l + 1, lc, lc, l // lc - 1, h))
    if hc < h:
        cor.append("\\multips(0,%s)(0,%s){%s}{\\psline[linecolor=Maroon](0,0)(%s,0)}" % 
                 (hc, hc, h // hc - 1, l))
        if cas == "nsd":
            cor.append("\\rput(%s,0){\\multips(0,%s)(0,%s){%s}{\\psline[linecolor=Maroon](0,0)(%s,0)}}" % 
                     (l + 1, hc, hc, h // hc - 1, l))


def coloriage(cor, n, d, l, h, lc, hc):
    if n == d:
        cor.append("\\psframe[fillstyle=solid](0,0)(%s,%s)" % 
                 (l, h))
    else:
        (x, y, nfig) = (0, 0, 0)
        for dummy in range(n):
            if nfig:
                cor.append("\\rput(%s,0){\\psframe[fillstyle=solid](%s,%s)(%s,%s)}" % 
                         (nfig, x, y, x + lc, y + hc))
            else:
                cor.append("\\psframe[fillstyle=solid](%s,%s)(%s,%s)" % 
                         (x, y, x + lc, y + hc))
            if x + lc < l:
                x = x + lc
            elif y + hc < h:
                (x, y) = (0, y + hc)
            else:
                (x, y, nfig) = (0, 0, l + 1)


def diviseurs(n):
    l = []
    for i in range(1, int(math.sqrt(n)) + 1):
        if not n % i:
            l.append(i)
            if i != n // i:
                l.append(n // i)
    l.sort()
    return l


def _FractionPartage():
    exo = ["\\exercice", '\\begin{multicols}{2}', '\\begin{enumerate}']
    cor = ["\\exercice*", '\\begin{multicols}{2}', '\\begin{enumerate}']

    lcas = ["nid", "un", "nsd", "nid", "nsd"]
    for i in range(4):
        cas = lcas.pop(random.randrange(len(lcas)))
        if cas == "nsd":
            while True:
                (l, h) = dimensions_rectangle()
                if l < 8:
                    break
        else:
            (l, h) = dimensions_rectangle()
        (n, d) = numerateur_denominateur(l, h, cas)
        (lc, hc) = fractions_partage_corrige(l, h, n, d)

        exo.append("\\item Colorer $\\frac{%s}{%s}$ de ce rectangle.\\par" % 
                 (n, d))
        cor.append("\\item Colorer $\\frac{%s}{%s}$ de ce rectangle.\\par" % 
                 (n, d))
        exo.append("\\psset{unit=4mm}")
        cor.append("\\psset{unit=4mm}")
        exo.append("\\begin{pspicture}(16,%s)" % h)
        cor.append("\\begin{pspicture}(16,%s)" % h)
        (lc, hc) = fractions_partage_corrige(l, h, n, d)
        coloriage(cor, n, d, l, h, lc, hc)
        trace_rectangle(exo, cor, l, h, cas)
        trace_partage(cor, l, h, lc, hc, cas)
        exo.append("\\end{pspicture}")
        cor.append("\\end{pspicture}")
        if i == 1:
            exo.append("\\columnbreak")
            cor.append("\\columnbreak")

    exo.append('\\end{enumerate}')
    exo.append('\\end{multicols}')
    cor.append('\\end{enumerate}')
    cor.append('\\end{multicols}')
    return (exo, cor)

class FractionPartage(LegacyExercise):
    """Fractions partage"""

    tags = ["Sixième"]
    function = _FractionPartage


#===============================================================================
# Fractions et abscisses
#===============================================================================


def valeurs_abscisses():
    origine = random.randrange(3, 11)
    nb_divisions = (
        6,
        8,
        9,
        10,
        12,
        14,
        15,
        16,
        18,
        20,
        )
    div = nb_divisions[random.randrange(len(nb_divisions))]
    nb_subd = diviseurs(div)
    subd = nb_subd[random.randrange(len(nb_subd) - 2) + 1]
    while subd < 3:
        subd = nb_subd[random.randrange(len(nb_subd) - 2) + 1]
    nb_grad = 58  # nb de graduations sur la demi-droite graduée
    lpts = [0 for i in range(7)]  # liste des places des points à trouver/placer sur la 1/2 droite graduée
    lpts[4] = random.randrange(1, nb_grad // div + 1) * div
    for i in range(2):
        a = random.randrange(1, nb_grad)
        while lpts.count(a):
            a = random.randrange(1, nb_grad)
        lpts[i] = a
    for i in range(2):
        a = random.randrange(1, (nb_grad * subd) // div)
        while lpts.count((a * div) // subd):
            a = random.randrange(1, (nb_grad * subd) // div)
        lpts[i + 2] = (a * div) // subd
    for i in range(2):
        a = random.randrange(1, (nb_grad * subd) // div)
        while lpts.count((a * div) // subd):
            a = random.randrange(1, (nb_grad * subd) // div)
        lpts[i + 5] = (a * div) // subd

    # npts=noms_pts(7)

    npts = ['A', 'B', 'C', 'D', 'E', 'F', 'G']
    lnum = list(range(7))  # liste des numérateurs
    lnum[0] = origine * div + lpts[0]
    lnum[1] = origine * div + lpts[1]
    lnum[2] = origine * subd + (lpts[2] * subd) // div
    lnum[3] = origine * subd + (lpts[3] * subd) // div
    lnum[4] = random.randrange(3, div)
    while diviseurs(div).count(lnum[4]):
        lnum[4] = random.randrange(3, div)
    lnum[5] = origine * div + lpts[5]
    lnum[6] = origine * div + lpts[6]
    return (origine, div, subd, lpts, npts, lnum)


def noms_pts(nb):  # renvoie nb noms de points
    (listenb, listepts) = ([], [])
    for i in range(26):
        listenb.append(i + 65)
    for i in range(nb):
        listepts.append(chr(listenb.pop(random.randrange(26 - i))))
    listepts.sort()
    return listepts


def unites_fractions(exo, cor, origine, div, subd):
    postf = u'ièmes'
    lch = [
        'cinqu',
        'six',
        'sept',
        'huit',
        'neuv',
        'dix',
        'onz',
        'douz',
        'treiz',
        'quatorz',
        'quinz',
        'seiz',
        'dix-sept',
        'dix-huit',
        'dix-neuv',
        'vingt',
        ]
    lfr = dict([(i + 5, lch[i] + postf) for i in range(len(lch))])
    lfr[2] = 'demis'
    lfr[3] = 'tiers'
    lfr[4] = 'quarts'
    exo.append(u'\\item 1 unité = \\ldots %s' % lfr[div])
    exo.append(u'\\item 1 unité = \\ldots~%s' % lfr[subd])
    exo.append(u'\\item %s unités = \\ldots~%s' % (origine,
             lfr[div]))
    exo.append(u'\\item %s unités = \\ldots~%s' % (origine,
             lfr[subd]))
    cor.append(u'\\item 1 unité = %s %s' % (div, lfr[div]))
    cor.append(u'\\item 1 unité = %s %s' % (subd, lfr[subd]))
    cor.append(u'\\item %s unités = %s %s' % (origine,
             origine * div, lfr[div]))
    cor.append(u'\\item %s unités = %s %s' % (origine,
             origine * subd, lfr[subd]))


def trace_demi_droite(exo, cor, origine, div, subd, lpts, npts, lnum):
    exo.append("\\psline[arrowscale=2,linecolor=Maroon]{->}(0,0)(18,0)")
    exo.append("\\rput(2mm,0){%")
    exo.append("\\multips(0,0)(3 mm,0){58}{\\psline[linecolor=Maroon](0,-.1)(0,.1)}")
    exo.append("\\multips(0,0)(%s mm,0){%s}{\\psline[linecolor=Maroon](0,-.2)(0,.2)}" % 
             (div * 3, 58 // div + 1))
    cor.append("\\psline[arrowscale=2,linecolor=Maroon]{->}(0,0)(18,0)")
    cor.append("\\rput(2mm,0){%")
    cor.append("\\multips(0,0)(3 mm,0){58}{\\psline[linecolor=Maroon](0,-.1)(0,.1)}")
    cor.append("\\multips(0,0)(%s mm,0){%s}{\\psline[linecolor=Maroon](0,-.2)(0,.2)}" % 
             (div * 3, 58 // div + 1))
    for i in range(58 // div + 1):
        exo.append("\\rput[t](%s mm,-3mm){\\centering %s}" % ((i * div) * 
                 3, origine + i))
        cor.append("\\rput[t](%s mm,-3mm){\\centering %s}" % ((i * div) * 
                 3, origine + i))
    for i in range(2):
        exo.append("\\rput[t](%s mm,4mm){\\centering $%s$}" % (lpts[i + 
                 5] * 3 + .1, npts[i + 5]))
    for i in range(7):
        cor.append("\\rput[t](%s mm,4mm){\\centering $%s$}" % (lpts[i] * 
                 3 + .1, npts[i]))
    exo.append("}")
    cor.append("}")


def ecrit_abscisses(exo, cor, origine, div, subd, lpts, lnum):
    exo.append("\\item $\\left(\\cfrac{%s}{%s}\\right)$" % (lnum[0], div))
    exo.append("\\item $\\left(\\cfrac{%s}{%s}\\right)$" % (lnum[1], div))
    exo.append("\\item $\\left(\\cfrac{%s}{%s}\\right)$" % (lnum[2], subd))
    exo.append("\\item $\\left(\\cfrac{%s}{%s}\\right)$" % (lnum[3], subd))
    exo.append("\\item $\\left(\\cfrac{%s}{%s}\\right)$" % (origine * 
             lnum[4] + (lpts[4] // div) * lnum[4], lnum[4]))
    cor.append("\\item $\\left(\\cfrac{%s}{%s}\\right)$" % (lnum[0], div))
    cor.append("\\item $\\left(\\cfrac{%s}{%s}\\right)$" % (lnum[1], div))
    cor.append("\\item $\\left(\\cfrac{%s}{%s}\\right)$" % (lnum[2], subd))
    cor.append("\\item $\\left(\\cfrac{%s}{%s}\\right)$" % (lnum[3], subd))
    cor.append("\\item $\\left(\\cfrac{%s}{%s}\\right)$" % (origine * 
             lnum[4] + (lpts[4] // div) * lnum[4], lnum[4]))


def trouve_abscisses(exo, cor, div, subd, lnum):
    exo.append("\\item $F~\\left(\\cfrac{\\ldots}{%s}\\right)$" % div)
    exo.append("\\item $F~\\left(\\cfrac{\\ldots}{%s}\\right)$" % subd)
    exo.append("\\item $G~\\left(\\cfrac{\\ldots}{%s}\\right)$" % div)
    exo.append("\\item $G~\\left(\\cfrac{\\ldots}{%s}\\right)$" % subd)
    cor.append("\\item $F~\\left(\\cfrac{%s}{%s}\\right)$" % (lnum[5], div))
    cor.append("\\item $F~\\left(\\cfrac{%s}{%s}\\right)$" % ((lnum[5] * 
             subd) // div, subd))
    cor.append("\\item $G~\\left(\\cfrac{%s}{%s}\\right)$" % (lnum[6], div))
    cor.append("\\item $G~\\left(\\cfrac{%s}{%s}\\right)$" % ((lnum[6] * 
             subd) // div, subd))


def _QuestionsAbscisses():
    exo = ["\\exercice"]
    cor = ["\\exercice*"]
    (origine, div, subd, lpts, npts, lnum) = valeurs_abscisses()
    exo.append("\\begin{enumerate}")
    exo.append(u"\\item Compléter :")
    exo.append("\\begin{multicols}{2}")
    exo.append("\\begin{enumerate}")
    cor.append("\\begin{enumerate}")
    cor.append(u"\\item Compléter :")
    cor.append("\\begin{multicols}{2}")
    cor.append("\\begin{enumerate}")
    unites_fractions(exo, cor, origine, div, subd)
    exo.append("\\end{enumerate}")
    exo.append("\\end{multicols}")
    exo.append(u"\\item Sur la demi-droite ci-dessous, placer les points d'abscisse donnée :")
    exo.append("\\begin{multicols}{5}")
    exo.append("\\begin{enumerate}")
    exo.append("\\renewcommand{\\theenumii}{\\Alph{enumii}}")
    exo.append("\\renewcommand{\\labelenumii}{$\\theenumii$}")
    cor.append("\\end{enumerate}")
    cor.append("\\end{multicols}")
    cor.append(u"\\item Sur la demi-droite ci-dessous, placer les points d'abscisse donnée :")
    cor.append("\\begin{multicols}{5}")
    cor.append("\\begin{enumerate}")
    cor.append("\\renewcommand{\\theenumii}{\\Alph{enumii}}")
    cor.append("\\renewcommand{\\labelenumii}{$\\theenumii$}")
    ecrit_abscisses(exo, cor, origine, div, subd, lpts, lnum)
    exo.append("\\end{enumerate}")
    exo.append("\\end{multicols}")
    exo.append(u"\\item Compléter les abscisses des points suivants :")
    exo.append("\\begin{multicols}{4}")
    exo.append("\\begin{enumerate}")
    cor.append("\\end{enumerate}")
    cor.append("\\end{multicols}")
    cor.append(u"\\item Compléter les abscisses des points suivants :")
    cor.append("\\begin{multicols}{4}")
    cor.append("\\begin{enumerate}")
    trouve_abscisses(exo, cor, div, subd, lnum)
    exo.append("\\end{enumerate}")
    exo.append("\\end{multicols}")
    exo.append("\\end{enumerate}")
    exo.append("\\begin{pspicture}(0,-.5)(18,.5)")
    cor.append("\\end{enumerate}")
    cor.append("\\end{multicols}")
    cor.append("\\end{enumerate}")
    cor.append("\\begin{pspicture}(0,-.5)(18,.5)")
    trace_demi_droite(exo, cor, origine, div, subd, lpts, npts, lnum)
    exo.append("\\end{pspicture}")
    cor.append("\\end{pspicture}")
    return (exo, cor)

class QuestionsAbscisses(LegacyExercise):
    """Fractions et abscisses"""

    tags = ["Sixième"]
    function = _QuestionsAbscisses



#!/usr/bin/env python3

import codecs
import collections
import difflib
import importlib
import inspect
import logging
import operator
import os
import pkgutil
import random
import shutil
import subprocess
import sys
import tempfile
import types

from ..outils import jinja2tex
from .test import test_path
from ..outils.System import Fiche
from .. import directories

class TexExercise:
    """Exercise with TeX support."""

    def __init__(self, seed=None):
        if seed is None:
            self.seed = random.randint(0, sys.maxsize)
        else:
            self.seed = seed
        random.seed(self.seed)

    @classmethod
    def name(cls):
        return cls.__name__

    @classmethod
    def description(cls):
        return cls.__doc__.splitlines()[0]

    @classmethod
    def thumb(cls):
        return os.path.join(directories.EXODIR, 'img', "%s.png" % cls.name())

    def tex_statement(self):
        """Return problem statement in TeX format."""
        raise NotImplementedError()

    def tex_answer(self):
        """Return full answer in TeX format."""
        raise NotImplementedError()

    ################################################################################
    # Tests

    def show(self, *, dir=None):
        """Generate exercise, and display its result."""
        subprocess.run(["gio", "open", self.generate(dir=dir)])

    def generate(self, *, enonce=True, corrige=True, dir=None):
        """Generate a single exercise.

        Return the pdf name.
        """
        parametres = {
            'enonce': enonce,
            'corrige': corrige,
            'exercices': [self],
            }
        output = os.path.join(
            tempfile.mkdtemp(dir=dir),
            "{}-{}.pdf".format(self.name(), self.seed),
            )
        with Fiche(parametres) as fiche:
            fiche.write_pdf()
            shutil.copy(fiche.pdfname, output)
        return output

    def test_path(self, name):
        """Return the path of the file containing expected results."""
        return test_path(
            self.name(),
            self.seed,
            name,
            )

    def write_test(self):
        """Write expected test results."""
        with codecs.open(self.test_path("statement"), "w", "utf8") as statement:
            statement.write(self.tex_statement())
        with codecs.open(self.test_path("answer"), "w", "utf8") as answer:
            answer.write(self.tex_answer())

    def read_test(self, choice):
        """Read expected test result."""
        with codecs.open(self.test_path(choice), "r", "utf8") as result:
            return result.read()

    def changed(self):
        """Return `True` iff exercise has changed."""
        if self.tex_statement() != self.read_test('statement'):
            return True
        if self.tex_answer() != self.read_test('answer'):
            return True
        return False

    def print_diff(self):
        """Print the diff between old and new test."""
        if self.tex_statement() != self.read_test('statement'):
            print("Statement:")
            for line in difflib.unified_diff(
                    self.read_test('statement').splitlines(),
                    self.tex_statement().splitlines(),
                    fromfile='Old statement',
                    tofile='New statement',
                    ):
                print(line)
        if self.tex_answer() != self.read_test('answer'):
            print("Answer:")
            for line in difflib.unified_diff(
                    self.read_test('answer').splitlines(),
                    self.tex_answer().splitlines(),
                    fromfile='Old answer',
                    tofile='New answer',
                    ):
                print(line)

class LegacyExercise(TexExercise):
    """Base class for legacy format exercise proxies.

    This class is deprecated. Do not use it to write new exercises.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.stat, self.ans = self.__class__.function()

    def tex_statement(self):
        return "\n".join(self.stat)

    def tex_answer(self):
        return "\n".join(self.ans)

def __import(name=__name__, parent=None):
    ''' Import 'name' from 'parent' package. '''
    if not isinstance(name, str):
        name = name.__name__
    # parent is None: assume 'name' is a package name
    # hack tout moche pour l'import des exercices dans la version Windows de Pyromaths :
    # Les modules sixiemes, quatriemes doivent être appelés avec le chemin complet,
    # alors que les exercices cinquiemes.aires ne doivent être appelés qu'ainsi.
    if "." not in name and hasattr(sys, "frozen"): name = "pyromaths.ex." + name
    if parent is None: parent = name
    elif not isinstance(parent, str):
        # assume 'parent' is a package instance
        parent = parent.__name__
    return importlib.import_module(name)
    # return __import__(name, fromlist=parent)

def _iter_pkg_exercises(pkg):
    ''' List exercises in 'pkg' modules. '''
    # level defaults to description, then unknown
    if 'level' not in dir(pkg): pkg.level = "Inconnu"
    for _, name, ispkg in pkgutil.iter_modules(pkg.__path__, pkg.__name__ + '.'):
        # skip packages
        if ispkg: continue;
        # import module
        mod = __import(name, pkg)
        # search exercises in module
        for name in dir(mod):
            cls = getattr(mod, name)
            if name.startswith("_"):
                continue
            if not isinstance(cls, type):
                continue
            if not issubclass(cls, TexExercise):
                continue
            if cls.__module__ == "pyromaths.ex":
                continue
            yield cls

def _subpackages(pkg):
    ''' List 'pkg' sub-packages. '''
    for _, name, ispkg in pkgutil.iter_modules(pkg.__path__, pkg.__name__ + '.'):
        # skip modules
        if not ispkg: continue;
        yield __import(name, pkg)

def _iter_exercises(pkg=None):
    """Iterator over existing exercises"""
    if pkg is None:
        pkg = __import()
    yield from _iter_pkg_exercises(pkg)

    # load sub-packages
    for sub in _subpackages(pkg):
        yield from _iter_exercises(sub)

NIVEAUX = [
    "Sixième",
    "Cinquième",
    "Quatrième",
    "Troisième",
    "Seconde",
    "1re Spé math",
    "Term STMG",
    "Term Spé math",
    # "exemple",
    ]

class ExerciseBag(collections.UserDict):
    """Classe regroupant tous les exercices.

    Cette classe est la sous-classe d'un dictionnaire :
    - clef: le nom (:type:`str`) des exercices ;
    - valeur: l'exercice (:type:`TexExercise`).

    Des méthodes permettent d'accéder à ces exercices avec d'autres présentations.
    """

    def __init__(self, *, exercises=None):
        super().__init__()
        if exercises is None:
            iterable = _iter_exercises()
        else:
            iterable = exercises
        for exo in iterable:
            if exo.name() in self:
                logging.error(
                    "Deux exercices portent le même nom '%s': %s et %s.",
                    exo.name(),
                    exo,
                    self[exo.name()],
                    )
            self[exo.name()] = exo

    def dict_levels(self):
        """Renvoit les exercices, comme un dictionnaire classé par niveau.

        - clefs: les niveaux ;
        - valeurs: l'ensemble des exercices de ce niveau.

        Remarque : Des exercices qui appartiennent à plusieurs niveaux peuvent
        apparaître plurieurs fois.
        """
        levels = collections.defaultdict(set)
        for exo in self.values():
            for lvl in exo.tags:
                levels[lvl].add(exo)
        return levels

    def sorted_levels(self):
        """Renvoit les exercices, sous la forme d'une liste triée par niveau.

        Chaque élément de la liste est lui-même une liste ``[niveau, exercices]``, où :
        - ``niveau`` : est le niveau (chaîne de caractères) ,
        - ``exercices`` : est la liste triée des exercices de ce niveau.

        Le tri par niveau est un tri logique pour un humain (sixième, puis cinquième, etc.).
        """
        levels = self.dict_levels()
        return [
            [niveau, sorted(levels[niveau], key=operator.methodcaller("name"))]
            for niveau in NIVEAUX
            ]

    def filter_tags(self, *clauses):
        if not clauses:
            return self
        exercises = set()
        for conjunction in clauses:
            for exo in self.values():
                if conjunction.issubset(set(exo.tags)):
                    exercises.add(exo)
        return self.__class__(exercises = exercises)

    def filter_desc(self, *descriptions):
        if not descriptions:
            return self
        exercises = set()
        for desc in descriptions:
            for exo in self.values():
                if desc.search(exo.description()):
                    exercises.add(exo)
        return self.__class__(exercises = exercises)

################################################################################
# Exercices créés à partir de templates Jinja2

def templatedir():
    return os.path.join(directories.EXODIR, "templates")

class Jinja2Exercise(TexExercise):
    """Exercice utilisant un template jinja2."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.context = {}

    @property
    def environment(self):
        """Création de l'environnement Jinja2, duquel sera chargé le template."""
        return jinja2tex.LatexEnvironment(
            loader=jinja2tex.FileSystemLoader(templatedir())
        )

    @property
    def statement_name(self):
        """Nom du fichier de l'énoncé (sans le répertoire)."""
        return os.path.join("{}-statement.tex".format(self.__class__.__name__))

    @property
    def answer_name(self):
        """Nom du fichier du corrigé (sans le répertoire)."""
        return os.path.join("{}-answer.tex".format(self.__class__.__name__))

    def tex_statement(self):
        """Génération de l'énoncé"""
        return self.environment.get_template(self.statement_name).render(self.context)

    def tex_answer(self):
        """Génération du corrigé"""
        return self.environment.get_template(self.answer_name).render(self.context)




#!/usr/bin/env python3
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

# from pyromaths.classes.Fractions import Fraction  # Fractions de pyromaths
import random
from pyromaths.outils import Arithmetique
from pyromaths.outils.Priorites3 import texify, priorites
from pyromaths.ex import LegacyExercise


def _fractions_egales():
    exo = ["\\exercice", _(u"Compléter :"), "\\begin{multicols}{4}",
           "  \\begin{enumerate}"]
    cor = ["\\exercice*", _(u"Compléter :"), "\\begin{multicols}{4}",
           "  \\begin{enumerate}"]
    for dummy in range(8):
        n = d = 1
        while n == d:
            n = random.randrange(1, 11)
            d = random.randrange(2, 11)
        c = random.randrange(2, 11)
        cas = random.randrange(2)
        if cas:
            enonce = [n, d, n * c, d * c]
            solution = [n, d, n * c, d * c]
        else:
            enonce = [n * c, d * c, n, d]
            solution = [n * c, d * c, n, d]
        trou = random.randrange(4)
        enonce[trou] = "\\ldots"
        solution[trou] = "\\mathbf{%s}" % solution[trou]
        if cas:
            solution.insert(2, c)
            solution.insert(1, c)
        else:
            solution.insert(4, c)
            solution.insert(3, c)
        exo.append("\\item $\\dfrac{%s}{%s}=\\dfrac{%s}{%s}$" %
                   tuple(enonce))
        if cas:
            cor.append("\\item $\\dfrac{%s_{(\\times %s)}}{%s_{(\\times %s)}}=\\dfrac{%s}{%s}$" %
                       tuple(solution))
        else:
            cor.append("\\item $\\dfrac{%s}{%s}=\\dfrac{%s_{(\\times %s)}}{%s_{(\\times %s)}}$" %
                       tuple(solution))
    exo.extend(["  \\end{enumerate}", "\end{multicols}\n"])
    cor.extend(["  \\end{enumerate}", "\end{multicols}\n"])
    return (exo, cor)

class fractions_egales(LegacyExercise):
    """Fractions égales"""

    tags = ["Cinquième"]
    function = _fractions_egales


def valeurs_somme():
    """Travail sur les sommes de fractions en Cinquième"""
    l = []

    for dummy in range(2):
        op = "+-"[random.randrange(2)]
        n2, d2 = random.randrange(1, 11), random.randrange(2, 11)
        if op == "-" and 1 - n2 // d2 > 0:
            l.append(_('1 %s Fraction(%s, %s)') % (op, n2, d2))
        else:
            l.append(_('Fraction(%s, %s) %s 1') % (n2, d2, op))

    for dummy in range(2):
        op = "+-"[random.randrange(2)]
        n1 = random.randrange(2, 11)
        n2, d2 = random.randrange(1, 11), random.randrange(2, 11)
        if op == "-" and n1 - n2 // d2 > 0:
            l.append(_('%s %s Fraction(%s, %s)') % (n1, op, n2, d2))
        else:
            l.append(_('Fraction(%s, %s) %s %s') % (n2, d2, op, n1))

    op = "+-"[random.randrange(2)]
    n1 = random.randrange(1, 11)
    n2, d2 = random.randrange(1, 11), random.randrange(2, 11)
    if op == "-" and n1 - n2 > 0:
        l.append(_('Fraction(%s, %s) %s Fraction(%s, %s)') % (n1, d2, op, n2, d2))
    else:
        l.append(_('Fraction(%s, %s) %s Fraction(%s, %s)') % (n2, d2, op, n1, d2))

    for dummy in range(3):
        op = "+-"[random.randrange(2)]
        n1, d1 = random.randrange(1, 11), random.randrange(2, 11)
        n2, d2 = random.randrange(1, 11), random.randrange(2, 11) * d1
        if op == "-" and n1 // d1 - n2 // d2 > 0:
            l.append(_('Fraction(%s, %s) %s Fraction(%s, %s)') % (n1, d1, op, n2, d2))
        else:
            l.append(_('Fraction(%s, %s) %s Fraction(%s, %s)') % (n2, d2, op, n1, d1))

    random.shuffle(l)
    return l


def _sommes_fractions():
    exo = [_("\\exercice"),
           _(u"Calculer en détaillant les étapes. Donner le résultat sous la forme d’une fraction la plus simple possible (ou d’un entier lorsque c’est possible)."),
           "\\begin{multicols}{4}", "  \\begin{enumerate}"]
    cor = [_("\\exercice*"),
           _(u"Calculer en détaillant les étapes. Donner le résultat sous la forme d’une fraction la plus simple possible (ou d’un entier lorsque c’est possible)."),
           "\\begin{multicols}{4}", "  \\begin{enumerate}"]
    lexo = valeurs_somme()
    for question in lexo:
        solve = [question]
        exo.append("\\item $\\thenocalcul = " + texify(solve)[0] + "$")
        cor.append("\\item $\\thenocalcul = " + texify(solve)[0] + "$")
        solve = priorites(question)
        solve = texify(solve)
        for e in solve:
            cor.append("\\[\\thenocalcul = " + e + "\\]")
        exo.append("\\stepcounter{nocalcul}")
        cor.append("\\stepcounter{nocalcul}")
    exo.extend(["  \\end{enumerate}", "\\end{multicols}"])
    cor.extend(["  \\end{enumerate}", "\\end{multicols}"])
    return (exo, cor)

class sommes_fractions(LegacyExercise):
    """Sommes de fractions"""

    tags = ["Cinquième"]
    function = _sommes_fractions


def valeurs_produit():
    l = []

    for dummy in range(4):
        n1 = d1 = n2 = d2 = a = b = 2
        while Arithmetique.pgcd(a, b) > 1:
            a = random.randrange(1, 11)
            b = random.randrange(2, 11)
        while Arithmetique.pgcd(n1 * a, d1 * b) > 1:
            n1 = random.randrange(1, 11)
            d1 = random.randrange(2, 11)
        while Arithmetique.pgcd(n2 * b, d2 * a) > 1:
            n2 = random.randrange(1, 11)
            d2 = random.randrange(2, 11)

        l.append(_('Fraction(%s, %s)*Fraction(%s,%s)') % (n1 * a, d1 * b, n2 * b, d2 * a))

    return l

def _produits_fractions():
    exo = ["\\exercice",
           _(u"Calculer en détaillant les étapes. Donner le résultat sous la forme d’une fraction la plus simple possible (ou d’un entier lorsque c’est possible)."),
           "\\begin{multicols}{4}", "  \\begin{enumerate}"]
    cor = ["\\exercice*",
           _(u"Calculer en détaillant les étapes. Donner le résultat sous la forme d’une fraction la plus simple possible (ou d’un entier lorsque c’est possible)."),
           "\\begin{multicols}{4}", "  \\begin{enumerate}"]
    lexo = valeurs_produit()
    for question in lexo:
        solve = [question]
        exo.append("\\item $\\thenocalcul = " + texify(solve)[0] + "$")
        cor.append("\\item $\\thenocalcul = " + texify(solve)[0] + "$")
        solve = priorites(question)
        solve = texify(solve)
        for e in solve:
            cor.append("\\[\\thenocalcul = " + e + "\\]")
        exo.append("\\stepcounter{nocalcul}")
        cor.append("\\stepcounter{nocalcul}")
    exo.extend(["  \\end{enumerate}", "\\end{multicols}"])
    cor.extend(["  \\end{enumerate}", "\\end{multicols}"])

    return (exo, cor)

class produits_fractions(LegacyExercise):
    """Produits de fractions"""

    tags = ["Cinquième"]
    function = _produits_fractions



#!/usr/bin/env python3
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
import random
from pyromaths.outils.decimaux import decimaux
from pyromaths.ex import LegacyExercise

#
# ------------------- Aire de disques -------------------
#


def arrondir(nombre):
    [partie_entiere, partie_decimale] = nombre.split(".")
    return int(partie_entiere) + (int(partie_decimale[0]) >= 5)


def _exo_aire_diques():
    exo = ["\\exercice"]
    cor = ["\\exercice*"]
    rayon1 = 2 * (random.randrange(33) + 1)
    rayon2 = int(1.5 * rayon1)
    i = random.randrange(2)
    if i == 0:
        donnees = (_('rayons'), rayon1, rayon2)
    else:
        donnees = (_(u'diamètres'), 2 * rayon1, 2 * rayon2)
    difference_des_carres = rayon2 ** 2 - rayon1 ** 2
    aire_arrondie = arrondir(str(3.14 * difference_des_carres))
    enonce = \
        _(u"""\\begin{minipage}{4cm}
\\begin{pspicture}(-2,-2)(2,2)
\\pscircle[fillstyle=solid](0,0){1.5}
\\pscircle[fillstyle=solid, fillcolor=white](0,0){1}
\\psdots[dotstyle=x](0,0)
\\rput(0.3;60){$O$}
\\end{pspicture}
\\end{minipage}\\hfill
\\begin{minipage}{13cm}
On considère deux cercles de centre $O$ et de %s respectifs $\\unit[%s]{cm}$ et $\\unit[%s]{cm}$.\\par
Calculer l'aire de la couronne circulaire (partie colorée) comprise entre les deux cercles en arrondissant le résultat au $\\unit{cm^2}$ le plus proche.""") % donnees
    exo.append(enonce)
    cor.append(enonce)
    cor.append(_("\\par\\dotfill{}\\\\\n"))
    if i == 0:
        cor.append(_(u"On calcule l'aire du disque de rayon $\\unit[%s]{cm}$:") % rayon2)
        cor.append(_(u"\\[\\pi \\times %s^2 = \\pi \\times %s \\times %s = \\unit[%s \\pi]{cm^2}\\]") % (rayon2, rayon2, rayon2, decimaux(rayon2 ** 2)))
        cor.append(_(u"On calcule l'aire du disque de rayon $\\unit[%s]{cm}$:") % rayon1)
        cor.append(_(u"\\[ \\pi \\times %s^2 = \\pi \\times %s \\times %s = \\unit[%s \\pi]{cm^2}\]") % (rayon1, rayon1, rayon1, decimaux(rayon1 ** 2)))
    else:
        cor.append(_(u"Un disque de diamètre $\\unit[%s]{cm}$ a pour rayon $%s \div 2 = \\unit[%s]{cm}$. Calculons son aire:") % (2 * rayon2, 2 * rayon2, rayon2))
        cor.append(_(u"\\[\\pi \\times %s^2 = \\pi \\times %s \\times %s = \\unit[%s \\pi]{cm^2}\\]") % (rayon2, rayon2, rayon2, decimaux(rayon2 ** 2)))
        cor.append(_(u"Un disque de diamètre $\\unit[%s]{cm}$ a pour rayon $%s \div 2 = \\unit[%s]{cm}$. Calculons son aire:") % (2 * rayon1, 2 * rayon1, rayon1))
        cor.append(_(u"\\[\\pi \\times %s^2 = \\pi \\times %s \\times %s = \\unit[%s \\pi]{cm^2}\\]") % (rayon1, rayon1, rayon1, decimaux(rayon1 ** 2)))
    cor.append(_(u"L'aire $\\mathcal{A}$ de la couronne est obtenue en retranchant l'aire du disque de rayon  $\\unit[%s]{cm}$  à l'aire du disque de rayon  $\\unit[%s]{cm}$:") % (rayon1, rayon2))
    cor.append(u"\\[\\mathcal{A} = %s \\pi  - %s \\pi= (%s - %s)\\pi =\\unit[%s \\pi]{cm^2}\\]" % (decimaux(rayon2 ** 2), decimaux(rayon1 ** 2), decimaux(rayon2 ** 2), decimaux(rayon1 ** 2), decimaux(difference_des_carres)))
    cor.append(_(u"L'aire exacte de la couronne est $\\unit[%s \\pi]{cm^2}$.") % (decimaux(difference_des_carres)))
    cor.append(_(u"En prenant 3,14 comme valeur approchée du nombre $\\pi$, on obtient :"))
    cor.append(u"\\[\\mathcal{A}  \\simeq %s \\times 3,14\\]" % decimaux(difference_des_carres))
    cor.append(u"\\[\\boxed{\\mathcal{A} \\simeq  \\unit[%s]{cm^2}}\\]" % decimaux(aire_arrondie))
    exo.append("\\end{minipage}\n")
    cor.append("\\end{minipage}\n")
    return (exo, cor)

class exo_aire_diques(LegacyExercise):
    """Aire de disques"""

    tags = ["Cinquième"]
    function = _exo_aire_diques



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from random import randrange
from pyromaths.classes.Polynome import Polynome, TeX, RacineDegre2
from pyromaths.classes.Racine import simplifie_racine
from pyromaths.outils.Priorites3 import priorites
from pyromaths.outils.Polynomes import poly_racines_entieres, poly_racines_fractionnaires, poly_racines_quelconques, poly_id_remarquables, poly_degre3_racines_entieres, TeX_division, poly_degre3_racines_fractionnaires, randint
# from pyromaths.outils.decimaux import decimaux
from pyromaths.outils.Affichage import pTeX, radicalTeX, fTeX
from pyromaths.outils.Arithmetique import pgcd
from pyromaths.ex import LegacyExercise
from math import sqrt



def _exo_racines_degre2():
    '''exercice recherche de racines second degré'''

    exo = ["\\exercice"]
    cor = ["\\exercice*"]
    # intervalle pour les racines entières ou fractionnaire
    rac_min = -10
    rac_max = 10
    # denominateur maximmum pour les racines fractionnaires
    denom_max = 12
    # Valeurs absolues maximales des coefficients d'un polynôme quelconque
    abs_a = 1
    abs_b = 10
    abs_c = 10
    # X est le polynome P=x pour faciliter la construction des polynômes,
    inconnues = ['x', 'y', 'z', 't']
#     nom_poly = ['P', 'Q', 'R', 'S']

    exo.append(_(u"Résoudre les équations suivantes :"))
    cor.append(_(u"Résoudre les équations suivantes :"))
    exo.append("\\begin{enumerate}")
    cor.append("\\begin{enumerate}")


    # Racines entières
    nomP = 'P'
    var = inconnues[randrange(4)]
    X = Polynome({1:1}, var)
    P = poly_racines_entieres(rac_min, rac_max, X)

    exo.append("\\item $%s=0$\\par" % (P(var)))
    cor.append("\\item $%s=0$\\par" % (P(var)))
    cor = redaction_racines(P, nomP, var, cor)

    # Racines fractionnaires
    nomP = 'P'
    var = inconnues[randrange(4)]
    X = Polynome({1:1}, var)
    P = poly_racines_fractionnaires(rac_min, rac_max, denom_max, X)

    exo.append("\\item $%s=0$\\par" % (P(var)))
    cor.append("\\item $%s=0$\\par" % (P(var)))
    cor = redaction_racines(P, nomP, var, cor)

    # Racines quelconques
    nomP = 'P'
    var = inconnues[randrange(4)]
    X = Polynome({1:1}, var)
    P = poly_racines_quelconques(abs_a, abs_b, abs_c, X)

    exo.append("\\item $%s=0$\\par" % (P(var)))
    cor.append("\\item $%s=0$\\par" % (P(var)))
    redaction_racines(P, nomP, var, cor)

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo, cor

class exo_racines_degre2(LegacyExercise):
    """Équations 2° degré"""

    tags = ["1re Spé math"]
    function = _exo_racines_degre2


def _exo_factorisation_degre2():
    '''exercice recherche de racines second degré'''

    exo = ["\\exercice"]
    cor = ["\\exercice*"]
    # intervalle pour les racines entières ou fractionnaire
    rac_min = -10
    rac_max = 10

    # X est le polynôme P(x)=x pour faciliter la construction des polynômes,
    inconnues = ['x', 'y', 'z', 't']
    nom_poly = ['P', 'Q', 'R', 'S']

    exo.append(_(u"Factoriser les polynômes suivants :"))
    # cor=[]u"Factoriser les polynômes suivants :"]
    exo.append("\\begin{enumerate}")
    cor.append("\\begin{enumerate}")

####identites remarquables

    nomP = nom_poly[randrange(4)]
    var = inconnues[randrange(4)]
    X = Polynome({1:1}, var)
    P, sgns = poly_id_remarquables(rac_min, rac_max, X)  # sgns=-2,0 ou 2
    exo.append(_(u"\\item Factoriser  $%s(%s)=%s$ à l'aide d'une identité remarquable.") % (nomP, var, P(var)))
    cor.append(_("\\item Factoriser $%s(%s)=%s$") % (nomP, var, P(var)))

    factorisation, dummy = factorise_identites_remarquables(P, sgns, var, racines=True)

    factorise = "$$%s" % P
    for i in range(len(factorisation)):
        factorise += "=" + factorisation[i]
    cor.append(factorise + "$$")

####Racines entières
    nomP = nom_poly[randrange(4)]
    var = inconnues[randrange(4)]
    X = Polynome({1:1}, var)
    P = poly_racines_entieres(rac_min, rac_max, X)

    exo.append("\\item $%s(%s)=%s$" % (nomP, var, P(var)))
    cor.append(_("\\item Factoriser $%s(%s)=%s$\\par") % (nomP, var, P(var)))
    exo, cor = redaction_factorisation(P, nomP, exo, cor)

####Racines fractionnaires
    nomP = nom_poly[randrange(4)]
    var = inconnues[randrange(4)]
    X = Polynome({1:1}, var)
    # denominateur maximmum pour les racines fractionnaires
    denom_max = 12
    P = poly_racines_fractionnaires(rac_min, rac_max, denom_max, X)

    exo.append("\\item $%s(%s)=%s$" % (nomP, var, P(var)))
    cor.append(_("\\item Factoriser $%s(%s)=%s$\\par") % (nomP, var, P(var)))
    exo, cor = redaction_factorisation(P, nomP, exo, cor)


####Racines quelconques
    nomP = nom_poly[randrange(4)]
    var = inconnues[randrange(4)]
    X = Polynome({1:1}, var)
    # Valeurs absolues maximales des coefficients d'un polynôme quelconque
    abs_a = 1
    abs_b = 10
    abs_c = 10
    P = poly_racines_quelconques(abs_a, abs_b, abs_c, X)

    exo.append("\\item $%s(%s)=%s$" % (nomP, var, P(var)))
    cor.append(_("\\item Factoriser $%s(%s)=%s$\\par") % (nomP, var, P(var)))
    exo, cor = redaction_factorisation(P, nomP, exo, cor)

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo, cor

class exo_factorisation_degre2(LegacyExercise):
    """Factorisations 2° degré"""

    tags = ["1re Spé math"]
    function = _exo_factorisation_degre2


def _exo_factorisation_degre3():
    '''exercice de factorisation degre3'''

    # intervalle pour les racines entières ou fractionnaire
    rac_min = -10
    rac_max = 10
    # denominateur maximmum pour les racines fractionnaires
    denom1 = 12
    # Valeurs absolues maximales des coefficients d'un polynôme quelconque
#     abs_a = 1
#     abs_b = 10
#     abs_c = 10
    # X est le polynôme P=x pour faciliter la construction des polynômes, TODO : changer  l'inconnue
#     inconnues = ['x', 'y', 'z', 't']
#     nom_poly = ['P', 'Q', 'R', 'S']
    exo = ["\\exercice",
        "\\begin{enumerate}"]
    cor = ["\\exercice*",
         "\\begin{enumerate}"]

    X = Polynome({1:1}, var="x")
    racines_quelconques = [i for i in range(-10, 11)]
    E = poly_degre3_racines_entieres(rac_min, rac_max, X, racines=racines_quelconques)
    exo, cor = factorisation_degre3(E, "E", exo, cor, racines=racines_quelconques)
    F = poly_degre3_racines_fractionnaires(rac_min, rac_max, denom1, X)
    exo, cor = factorisation_degre3(F, "F", exo, cor)

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo, cor

class exo_factorisation_degre3(LegacyExercise):
    """Factorisations degré 3"""

    tags = ["1re Spé math"]
    function = _exo_factorisation_degre3


def _exo_tableau_de_signe():
    # intervalle pour les racines entières ou fractionnaire
    rac_min = -10
    rac_max = 10
    # denominateur maximmum pour les racines fractionnaires
    denom1 = 12
    # Valeurs absolues maximales des coefficients d'un polynôme quelconque
    abs_a = 1
    abs_b = 10
    abs_c = 10
    # X est le polynôme P=x pour faciliter la construction des polynômes, TODO : changer  l'inconnue
#     inconnues = ['x', 'y', 'z', 't']
#     nom_poly = ['P', 'Q', 'R', 'S']
    borneinf = -5  # float("-inf")
    bornesup = 5  # float("inf")
    var = "x"
    X = Polynome({1:1}, var=var)
    Poly = [poly_racines_entieres(rac_min, rac_max, X),
          poly_racines_fractionnaires(rac_min, rac_max, denom1, X),
          poly_racines_quelconques(abs_a, abs_b, abs_c, X)]
    intervalles = [[0, 5], [-5, 5], [float("-inf"), float("inf")]]
    exo = ["\\exercice",
        "\\begin{enumerate}"]
    cor = ["\\exercice*",
         "\\begin{enumerate}"]
    nomP = "P"
    for i in range(len(Poly)):
        P = Poly[i]
        borneinf, bornesup = intervalles[i]
        if borneinf == float("-inf") and bornesup == float("inf"):
            TeXintervalle = "\\mathbb R"
        else:
            if borneinf != float("-inf"):
                TeXintervalle = "["
            else:
                TeXintervalle = "]"
            TeXintervalle += "%s~;~%s" % (TeX(borneinf), TeX(bornesup))
            if bornesup == float("inf"):
                TeXintervalle += "["
            else:
                TeXintervalle += "]"
        exo.append(_(u"\\item Étudier le signe du polynôme $%s=%s$ sur $I=%s$.") % (nomP, P, TeXintervalle))
        cor.append(_(u"\\item Étudier le signe du polynôme $%s=%s$ sur $I=%s$.\\par") % (nomP, P, TeXintervalle))

        delta, dummy, racines, dummy, dummy = factorisation_degre2(P, factorisation=False)
        redaction_racines(P, nomP, var, cor)
        tableau_de_signe(P, nomP, delta, racines, cor, borneinf, bornesup, detail=False)
    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo, cor

class exo_tableau_de_signe(LegacyExercise):
    """Étude de signe"""

    tags = ["1re Spé math"]
    function = _exo_tableau_de_signe


def _exo_variation():

    exo = ["\\exercice",
         "\\begin{enumerate}"]
    cor = ["\\exercice*",
         "\\begin{enumerate}"]
    quest1, cor1 = quest_fonctions_rationnelles()
    quest2, cor2 = quest_variation_degre3(borneinf=-10, bornesup=10)

    exo += quest1 + quest2
    cor += cor1 + cor2

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo, cor

class exo_variation(LegacyExercise):
    """Sens de variations"""

    tags = ["1re Spé math", "Term STMG"]
    function = _exo_variation


def _exo_variation_lim():
    """Étude de fonctions avec calculs de limites"""
    exo = ["\\exercice",
         "\\begin{enumerate}"]
    cor = ["\\exercice*",
         "\\begin{enumerate}"]
    quest3, cor3 = quest_variation_degre3(borneinf=float("-inf"), bornesup=float("+inf"))
    quest4, cor4 = quest_fonctions_rationnelles_sur_R()
    exo += quest3 + quest4
    cor += cor3 + cor4

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo, cor

class exo_variation_lim(LegacyExercise):
    """Étude de fonctions"""

    tags = ["Term Spé math"]
    function = _exo_variation_lim

def quest_fonctions_rationnelles():
    from pyromaths.classes.Fractions import Fraction
    from pyromaths.outils import Priorites3

    nomf = ['f', 'g', 'h', 'k'][randrange(4)]
    var = ['t', 'x'][randrange(2)]
    X = Polynome({1:1}, var)
    # intervalle pour les racines entières ou fractionnaire

    rac_min = -9
    rac_max = 9
    b1 = b2 = a1 = a2 = 0
    while b1 == 0 or b2 == 0 or a1 == 0 or a2 == 0:
        b1 = randint(rac_min, rac_max)
        b2 = randint(rac_min, rac_max)
        a1 = randint(-5, 5)
        a2 = randint(-5, 5)
    P = a1 * X + b1
    Q = a2 * X + b2

    borneinf = -10
    bornesup = 10
    racine = eval(Priorites3.priorites("-%r/%r" % (Q[0], Q[1]))[-1][0])
    # Je veux que f soit définie et dérivable sur I=Intervalle
    if (racine >= borneinf) and (racine <= bornesup):
        if (racine - borneinf) < (bornesup - racine):
            Intervalle = [int(round(racine)) + 1, bornesup]
        else:
            Intervalle = [borneinf, int(round(racine)) - 1]
    else:
        Intervalle = [borneinf, bornesup]

    # dérivée
    numerateur = "%s\\times{}%s-%s\\times{}%s" % (P.derive().TeX(parenthese=True), Q.TeX(parenthese=True),
                                          P.TeX(parenthese=True), Q.derive().TeX(parenthese=True))
    numerateur_simplifie = (P.derive() * Q - P * Q.derive()).simplifie()
    denominateur = u"%s^2" % (Q.TeX(parenthese=True))
    f_derivee = "\\dfrac{%s}{%s}" % (numerateur, denominateur)
    f_derivee_simplifiee = "\\dfrac{%s}{%s}" % (numerateur_simplifie, denominateur)
    VI = eval(priorites('-%r*Fraction(1)/%r' % (Q[0], Q[1]))[-1][0])
    if isinstance(VI, (Fraction, RacineDegre2)): VI = VI.simplifie()
    exo = [_(u"\\item On considère la fonction $%s$ définie sur $I=[%s~;~%s]$ par $%s(%s)=\dfrac{%s}{%s}$.") % (nomf, Intervalle[0], Intervalle[1], nomf, var, P, Q),
         "\\begin{enumerate}"]
    cor = [_(u"\\item On considère la fonction $%s$ définie sur $I=[%s~;~%s]$ par $%s(%s)=\dfrac{%s}{%s}$.") % (nomf, Intervalle[0], Intervalle[1], nomf, var, P, Q),
         "\\begin{enumerate}"]

    exo.append(_(u"\\item Justifier que $%s$ est définie et dérivable sur $I$.") % (nomf))
    cor.append(_(u"\\item Justifier que $%s$ est définie et dérivable sur $I$.") % (nomf))
    cor.append(_(u" Pour déterminer la valeur interdite, on doit résoudre $%s=0$.") % (Q(var)))
    cor.append("\\begin{align*}")
    cor.append("%s&=0\\\\" % Q)
    cor.append("%s&=%s" % ((Q - Q[0]), TeX(-Q[0])))
    cor.append("\\\\")
    if Q[1] != 1:
        x0 = eval(priorites('-%r*Fraction(1)/%r' % (Q[0], Q[1]))[-1][0])
        if isinstance(x0, (Fraction, RacineDegre2)):
            x1 = x0.simplifie()
        else:
            x1 = x0
        cor.append("%s&=%s" % (var, Fraction(-Q[0], Q[1])))
        cor.append("\\\\")
        if isinstance(x0, (Fraction, RacineDegre2)) and (not isinstance(x1, (Fraction, RacineDegre2)) or x0.d != x1.d):
            cor.append("%s&=%s" % (var, TeX(x1)))
            cor.append("\\\\")
    cor.pop(-1)
    cor.append("\\end{align*}")
    cor.append(_(u"Or $%s$ n'est pas dans l'intervalle $[%s~;~%s]$ et comme $%s$ est un quotient de polynômes, alors $%s$ est définie et dérivable sur $I$.") % \
               (TeX(VI), Intervalle[0], Intervalle[1], nomf, nomf))
    exo.append(_(u"\\item Déterminer $%s'(%s)$ pour tout $%s\in[%s~;~%s]$.") % \
          (nomf, var, var, Intervalle[0], Intervalle[1]))
    cor.append(_(u"\\item Déterminer $%s'(%s)$ pour tout $%s\in[%s~;~%s]$.") % \
          (nomf, var, var, Intervalle[0], Intervalle[1]))
    cor.append(u"$$%s'(%s)=%s=%s$$" % (nomf, var, f_derivee, f_derivee_simplifiee))
    exo.append(_(u"\\item En déduire le sens de variations de $%s$ sur $I$.") % (nomf))
    cor.append(_(u"\\item En déduire le sens de variations de $%s$ sur $I$.\\par") % (nomf))
    if numerateur_simplifie.degre_max == 0:
        cor.append(_(u" Comme $%s$ est un carré, il est toujours positif.\\\\") % (denominateur))
        f_xmin = eval(priorites('%r*Fraction(1)/%r' % (P(Intervalle[0]), Q(Intervalle[0])))[-1][0])
        f_xmax = eval(priorites('%r*Fraction(1)/%r' % (P(Intervalle[1]), Q(Intervalle[1])))[-1][0])
        if isinstance(f_xmin, (Fraction, RacineDegre2)): f_xmin = f_xmin.simplifie()
        if isinstance(f_xmax, (Fraction, RacineDegre2)): f_xmax = f_xmax.simplifie()
        f_xmin = TeX(f_xmin)
        f_xmax = TeX(f_xmax)
        if numerateur_simplifie[0] < 0:
            cor.append(_(u" De plus, $%s<0$ donc pour tout $%s$ de $I$, $%s'(%s)<0$. Ainsi, on obtient \\par") % \
                  (numerateur_simplifie[0], var, nomf, var))
            cor.append("\\begin{tikzpicture}\n\\tkzTabInit[espcl=2.5]")
            cor.append("{$%s$ /1, $%s'\\,(x)$/1, $%s\\,(x)$/1.5}" % (var, nomf, nomf))
            cor.append("{$%s$,$%s$}" % (TeX(Intervalle[0]), TeX(Intervalle[1])))
            cor.append("\\tkzTabLine{,-,}")
            cor.append("\\tkzTabVar{+/$%s$/, -/$%s$/}" % (f_xmin, f_xmax))
            cor.append(r'\end{tikzpicture}\par')
            #===================================================================
            # cor.append("$$\\tabvar{")
            # cor.append("\\tx{%s}&\\tx{%s}&&\\tx{%s}\\cr" % (var, TeX(Intervalle[0]), TeX(Intervalle[1])))
            # cor.append("\\tx{%s'(%s)}&&\\tx{-}&&\\cr" % (nomf, var))
            # cor.append("\\tx{%s}&\\txh{%s}&\\fd&\\txb{%s}\\cr" % (nomf, f_xmin, f_xmax))
            # cor.append("}$$")
            #===================================================================
        else:
            cor.append(_(u" De plus, $%s>0$ donc pour tout $%s$ de $I$, $%s'(%s)>0$.\\par") % \
                  (numerateur_simplifie[0], var, nomf, var))
            cor.append("\\begin{tikzpicture}\n\\tkzTabInit[espcl=2.5]")
            cor.append("{$%s$ /1, $%s'\\,(x)$/1, $%s\\,(x)$/1.5}" % (var, nomf, nomf))
            cor.append("{$%s$,$%s$}" % (TeX(Intervalle[0]), TeX(Intervalle[1])))
            cor.append("\\tkzTabLine{,+,}")
            cor.append("\\tkzTabVar{-/$%s$/, +/$%s$/}" % (f_xmin, f_xmax))
            cor.append(r'\end{tikzpicture}\par')
            #===================================================================
            # cor.append("$$\\tabvar{")
            # cor.append("\\tx{%s}&\\tx{%s}&&\\tx{%s}\\cr" % (var, TeX(Intervalle[0]), TeX(Intervalle[1])))
            # cor.append("\\tx{%s'(%s)}&&\\tx{+}&&\\cr" % (nomf, var))
            # cor.append("\\tx{%s}&\\txb{%s}&\\fm&\\txh{%s}\\cr" % (nomf, f_xmin, f_xmax))
            # cor.append("}$$")
            #===================================================================
    else:
        cor.append(_(u" Je ne sais pas faire avec un tel numérateur $%s$.") % (numerateur_simplifie))
    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo, cor

def quest_fonctions_rationnelles_sur_R():
    from pyromaths.classes.Fractions import Fraction

    nomf = ['f', 'g', 'h', 'k'][randrange(4)]
    var = ['t', 'x'][randrange(2)]
    X = Polynome({1:1}, var)
    # intervalle pour les racines entières ou fractionnaire

    rac_min = -9
    rac_max = 9
    b1 = b2 = a1 = a2 = 0
    while b1 == 0 or b2 == 0 or a1 == 0 or a2 == 0 or a1 * -b2 / a1 + b1 == 0 or (a1 * b2 / a2 - b1) == 0:
        # (a1*b2/a2 - b1)==0 on teste que la racine de Q n'annule pas P donc on ne peut pas simplifier
        b1 = randint(rac_min, rac_max)
        b2 = randint(rac_min, rac_max)
        a1 = randint(-5, 5)
        a2 = randint(-5, 5)
    P = a1 * X + b1
    Q = a2 * X + b2

    borneinf = float("-inf")
    bornesup = float("+inf")
    Intervalle = [borneinf, bornesup]
#     TeXintervalle = "\\mathbb R"

    # dérivée
    numerateur = "%s\\times{}%s-%s\\times{}%s" % (P.derive().TeX(parenthese=True), Q.TeX(parenthese=True),
                                          P.TeX(parenthese=True), Q.derive().TeX(parenthese=True))
    numerateur_simplifie = (P.derive() * Q - P * Q.derive()).simplifie()
    # VI = (-Q[0] * Fraction(1) / Q[1]).simplifie()
    #===========================================================================
    # print "A simplifier : ", priorites('-%r*Fraction(1)/%r' % (Q[0], Q[1]))[-1][0]
    #===========================================================================
    VI = eval(priorites('-%r*Fraction(1)/%r' % (Q[0], Q[1]))[-1][0])
    if isinstance(VI, (Fraction, RacineDegre2)): VI.simplifie()
    denominateur = u"%s^2" % (Q.TeX(parenthese=True))
    f_derivee = "\\dfrac{%s}{%s}" % (numerateur, denominateur)
    f_derivee_simplifiee = "\\dfrac{%s}{%s}" % (numerateur_simplifie, denominateur)

    exo = [_(u"\\item On considère la fonction $%s$ définie  par $%s(%s)=\\dfrac{%s}{%s}$.") % (nomf, nomf, var, P, Q),
         "\\begin{enumerate}"]
    cor = [_(u"\\item On considère la fonction $%s$ définie  par $%s(%s)=\\dfrac{%s}{%s}$.") % (nomf, nomf, var, P, Q),
         "\\begin{enumerate}"]

    exo.append(_(u"\\item Déterminer l'ensemble de définition $\\mathcal{D}_{%s}$ de $%s$.") % (nomf, nomf))
    cor.append(_(u"\\item Déterminer l'ensemble de définition $\\mathcal{D}_{%s}$ de $%s$.\par") % (nomf, nomf))
    cor.append(_(u" La fonction rationnelle $%s$ est définie et dérivable en $%s$ si $%s\\neq0$.") % (nomf, var, Q(var)))
    if Q[1] == 1:
        cor.append("\\begin{align*}\n\
                %s&=0\\\\\n\
                %s&=%s\\\\\n\
                \\end{align*}" % (Q, (Q - Q[0]), TeX(-Q[0])))
    else:
        cor.append("\\begin{align*}\n\
                %s&=0\\\\\n\
                %s&=%s\\\\\n\
                %s&=%s\\\\\n\
                %s&=%s\n\
                \\end{align*}" % (Q, (Q - Q[0]), TeX(-Q[0]), var, TeX(Fraction(-Q[0],1)) + "\\times " + TeX(Fraction(1,Q[1])), var, TeX(VI)))
    cor.append(_(u"On en déduit que $\\mathcal{D}_{%s}=\\mathcal{D'}_{%s}=]-\\infty~;~%s[\cup]%s~;~+\\infty[$.") % \
          (nomf, nomf, TeX(VI), TeX(VI)))
    exo.append(_(u"\\item Déterminer $%s'(%s)$ pour tout $%s\in\\mathcal{D'}_{%s}$.") % \
          (nomf, var, var, nomf))
    cor.append(_(u"\\item Déterminer $%s'(%s)$ pour tout $%s\in\\mathcal{D'}_{%s}$.") % \
          (nomf, var, var, nomf))
    cor.append(u"$$%s'(%s)=%s=%s$$" % (nomf, var, f_derivee, f_derivee_simplifiee))
    exo.append(_(u"\\item Déterminer les limites de $%s$ aux bornes de $\\mathcal{D}_{%s}$.") % (nomf, nomf))
    cor.append(_(u"\\item Déterminer les limites de $%s$ aux bornes de $\\mathcal{D}_{%s}$.") % (nomf, nomf))
    #===========================================================================
    # limite = P[1] / Q[1]
    #===========================================================================
    limite = eval(priorites('-%r*Fraction(1)/%r' % (P[1], Q[1]))[-1][0])
    if isinstance(limite, (Fraction, RacineDegre2)):
        limite_simple = limite.simplifie()
    else:
        limite_simple = limite

    cor.append("$$\\lim_{%s\\to -\\infty}\\dfrac{%s}{%s}= " % (var, P, Q))

    cor.append("\\lim_{%s\\to -\\infty}\\dfrac{%s%s}{%s%s} = %s" % (var, coeffTeX(P[1]), var, coeffTeX(Q[1]), var, limite))
    if isinstance(limite, (int, float)) or (isinstance(limite_simple, (Fraction, RacineDegre2)) and limite.n == limite_simple.n):
        cor.append("$$")
    else:
        cor.append("= %s $$" % (TeX(limite_simple)))
    cor.append("$$\\lim_{%s\\to +\\infty}\\dfrac{%s}{%s}= " % (var, P, Q))
    cor.append("\\lim_{%s\\to +\\infty}\\dfrac{%s%s}{%s%s} = %s" % (var, coeffTeX(P[1]), var, coeffTeX(Q[1]), var, limite))
    if isinstance(limite, (int, float)) or (isinstance(limite_simple, (Fraction, RacineDegre2)) and limite.n == limite_simple.n):
        cor.append("$$")
    else:
        cor.append("= %s $$" % (TeX(limite_simple)))
    cor.append("Pour $%s=%s$, on a $%s=%s" % (var, TeX(VI), P, TeX(P(VI))))
    if P(VI) < 0:
        limites = ["-\\infty", "+\\infty"]
        cor.append("<0$.\\\\")
    elif P(VI) > 0:
        limites = ["+\\infty", "-\\infty"]
        cor.append(">0$.\\\\")
    else:cor.append("$.\\\\")
        # Impossible car on test (a1*b2/a2 - b1)!=0

    VIplus = "\\substack{%s\\to %s\\\\%s>%s}" % (var, fTeX(VI), var, fTeX(VI))
    VImoins = "\\substack{%s\\to %s\\\\%s<%s}" % (var, fTeX(VI), var, fTeX(VI))
    if Q[1] < 0:
        cor.append(_("De plus, $%s>0$ si $%s<%s$") % (Q, var, TeX(VI)))
        cor.append(_("et  $%s<0$ si $%s>%s$.\\\\") % (Q, var, TeX(VI)))
    else:
        cor.append(_("De plus, $%s<0$ si $%s<%s$") % (Q, var, TeX(VI)))
        cor.append(_("et  $%s>0$ si $%s>%s$.\\\\") % (Q, var, TeX(VI)))

    cor.append(u"$$\\lim_{%s}\\dfrac{%s}{%s}=%s $$" % (VImoins, P, Q, limites[0]))
    cor.append(u"$$\\lim_{%s}\\dfrac{%s}{%s}=%s $$" % (VIplus, P, Q, limites[1]))



    exo.append(_(u"\\item Dresser le tableau de variations de $%s$ sur $\\mathcal{D}_{%s}$.") % (nomf, nomf))
    cor.append(_(u"\\item Dresser le tableau de variations de $%s$ sur $\\mathcal{D}_{%s}$.\\par") % (nomf, nomf))
    if numerateur_simplifie.degre_max == 0:
        cor.append(_(u" Comme $%s$ est un carré, il est toujours positif.\\\\") % (denominateur))
        f_xmin = eval(priorites('-%r*Fraction(1)/%r' % (P[1], Q[1]))[-1][0])
        if isinstance(f_xmin, (Fraction, RacineDegre2)):
            f_xmin = f_xmin.simplifie()
        f_xmax = f_xmin
        if numerateur_simplifie[0] < 0:
            cor.append(_(u" De plus, $%s<0$ donc pour tout $%s$ de $I$, $%s'(%s)<0$. Ainsi, on obtient \\par") % \
                  (numerateur_simplifie[0], var, nomf, var))
            cor.append("\\begin{tikzpicture}\n\\tkzTabInit[espcl=2.5]")
            cor.append("{$%s$ /1, $%s'\\,(x)$/1, $%s\\,(x)$/1.5}" % (var, nomf, nomf))
            cor.append("{$%s$,$%s$,$%s$}" % (TeX(Intervalle[0]), TeX(VI), TeX(Intervalle[1])))
            cor.append("\\tkzTabLine{,-,d,-}")
            cor.append("\\tkzTabVar{+/$%s$ / ,-D+/ $-\\infty$ /$+\infty$, -/$%s$/}" % (f_xmin, f_xmax))
            cor.append(r'\end{tikzpicture}\par')
            #===================================================================
            # cor.append("$$\\tabvar{")
            # cor.append("\\tx{%s}&\\tx{%s}&&&\\tx{%s}&&&\\tx{%s}\\cr" % (var, TeX(Intervalle[0]), TeX(VI), TeX(Intervalle[1])))
            # cor.append("\\tx{%s'(%s)}&&\\tx{-}&&\\dbt &&\\tx{-}&\\cr" % (nomf, var))
            # cor.append("\\tx{%s}&\\txh{%s}&\\fd&\\txb{-\\infty}&\\dbt&\\txh{+\\infty}&\\fd&\\txb{%s}\\cr" % (nomf, f_xmin, f_xmax))
            # cor.append("}$$")
            #===================================================================
        else:
            cor.append(_(u" De plus, $%s>0$ donc pour tout $%s$ de $I$, $%s'(%s)>0$. \\par") % \
                  (numerateur_simplifie[0], var, nomf, var))
            cor.append("\\begin{tikzpicture}\n\\tkzTabInit[espcl=2.5]")
            cor.append("{$%s$ /1, $%s'\\,(x)$/1, $%s\\,(x)$/1.5}" % (var, nomf, nomf))
            cor.append("{$%s$,$%s$,$%s$}" % (TeX(Intervalle[0]), TeX(VI), TeX(Intervalle[1])))
            cor.append("\\tkzTabLine{,+,d,+}")
            cor.append("\\tkzTabVar{-/$-\\infty$ / ,+D-/ $%s$ /$%s$, +/$+\\infty$/}" % (f_xmin, f_xmax))
            cor.append(r'\end{tikzpicture}\par')
            #===================================================================
            # cor.append("$$\\tabvar{")
            # cor.append("\\tx{%s}&\\tx{%s}&&&\\tx{%s}&&&\\tx{%s}\\cr" % (var, TeX(Intervalle[0]), TeX(VI), TeX(Intervalle[1])))
            # cor.append("\\tx{%s'(%s)}&&\\tx{+}&&\\dbt&&\\tx{+}&\\cr" % (nomf, var))
            # cor.append("\\tx{%s}&\\txb{%s}&\\fm&\\txh{+\\infty}&\\dbt&\\txb{-\\infty}&\\fm&\\txh{%s}\\cr" % (nomf, f_xmin, f_xmax))
            # cor.append("}$$")
            #===================================================================
    else:
        cor.append(_(u" Je ne sais pas faire avec un tel numérateur $%s$.") % (numerateur_simplifie))
    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo, cor

def coeffTeX(a):
        if a == 1:return ""
        if a == -1:return "-"
        return a

def quest_variation_degre3(borneinf=float("-inf"), bornesup=float("+inf")):
    '''Question qui propose l'étude du sens de variation d'un polynôme de degré 3'''
#     Intervalle = [borneinf, bornesup]
    if borneinf == float("-inf") and bornesup == float("+inf"):
        TeX_intervalle = "\\mathbb R"
    else:
        TeX_intervalle = "\\left[%s~;~%s\\right]" % (TeX(borneinf), TeX(bornesup))
    # intervalle pour les racines entières ou fractionnaire
    a = 3 * randint(1, 3)
    rac_min = -9
    rac_max = 9
    # denominateur maximmum pour les racines fractionnaires
#     denom_max = denom1 = 12
    # Valeurs absolues maximales des coefficients d'un polynôme quelconque
#     abs_a = 6
#     abs_b = 10
    abs_c = 10
    # X est le polynôme P=x pour faciliter la construction des polynômes,
#     inconnues = ['x', 'y', 'z', 't']
#     nom_poly = ['P', 'Q', 'R', 'S']
    var = "x"
    X = Polynome({1:1}, var=var)
    nomP = ["f", "g", "h", "k", "p", "q"][randrange(6)]
    Pprime = poly_racines_entieres(rac_min, rac_max, X, a1=a)
    P = Pprime.primitive() + randint(-abs_c, abs_c)
    P = P.simplifie()
    exo = [_(u"\\item Étudier le sens de variations de $%s$ définie par $%s(x)=%s$ sur $%s$.") % (nomP, nomP, P(var), TeX_intervalle)]
    cor = [_(u"\\item Étudier le sens de variations de $%s$ définie par $%s(x)=%s$ sur $%s$.") % (nomP, nomP, P(var), TeX_intervalle)]

    cor.append("\\par $%s'(x)=%s$\\\\" % (nomP, Pprime(var)))
    cor.append(_(u"Je dois étudier le signe de $%s'(%s)$ qui est un polynôme du second degré.\\par") % (nomP, var))

    delta, simplrac, racines, str_racines, factorisation = factorisation_degre2(Pprime, factorisation=False)
    # cor=redaction_factorisation(Pprime,nomP+"'",exo=[],cor=cor)[1]
    # cor.pop(-5)
    redaction_racines(Pprime, nomP + "'", var, cor)
    str_variables, str_signes, str_valeurs, signes, ligne_valeurs = tableau_de_signe(Pprime, nomP + "'", delta, racines, cor, borneinf, bornesup, detail=True)

    # cor.append(tab_signe)


    if (delta <= 0 and P[3] < 0):
        cor += _(u"Donc la fonction polynômiale $%s$ est décroissante sur $%s$.") % (nomP, TeX_intervalle)
    elif (delta <= 0 and P[3] > 0):
        cor.append(_(u"Donc la fonction polynômiale $%s$ est croissante sur $%s$.") % (nomP, TeX_intervalle))
    else:
        cor.append(_("On obtient ainsi le tableau de variation de $%s$.\\par") % nomP)
#         [x1, x2] = racines
        # macro=[["txb","txh"],["fm","fd"]]
        cor.append("\\begin{tikzpicture}\n\\tkzTabInit[espcl=2.5]")
        cor.append(str_variables[:-1] + ', $%s\\,(x)$/1.5}' % nomP)
        cor.append(str_valeurs)
        cor.append(str_signes)
        var_de_P = "\\tkzTabVar{%s/$%s$, " % (["-", "+"]["-" == signes[0]], TeX(P(ligne_valeurs[0])))  # +/$%s$/, -/$%s$/}" % (f_xmin, f_xmax))
        #=======================================================================
        # var_de_P = "\\tx{%s}& \\%s{\\rnode{neu0}{%s}}&&" % (nomP, ["txb", "txh"]["-" == signes[0]], TeX(P(ligne_valeurs[0])))
        #=======================================================================
        compteur = 0
        for i in range(0, len(signes) - 1):
            if signes[i] == '+':
                if 0 and signes[i + 1] == "+":
                    var_de_P += "&&"
                else:
                    compteur += 1
                    var_de_P += "+/$%s$/, " % TeX(P(ligne_valeurs[i + 1]))
                    #===========================================================
                    # var_de_P += "\\txh{\\rnode{neu%s}{%s}}&&" % (compteur, TeX(P(ligne_valeurs[i + 1])))
                    #===========================================================
            else:
                if 0 and signes[i + 1] == "-":
                    var_de_P += "&&"
                else:
                    compteur += 1
                    var_de_P += "-/$%s$/, " % TeX(P(ligne_valeurs[i + 1]))
                    #===========================================================
                    # var_de_P += "\\txb{\\rnode{neu%s}{%s}}&&" % (compteur, TeX(P(ligne_valeurs[i + 1])))
                    #===========================================================
        compteur += 1
        if signes[-1] == "+":
            var_de_P += "+/$%s$/} " % TeX(P(ligne_valeurs[-1]))
            #===================================================================
            # var_de_P += "\\txh{\\rnode{neu%s}{%s}}\\cr" % (compteur, TeX(P(ligne_valeurs[-1])))
            #===================================================================
        else:
            var_de_P += "-/$%s$/} " % TeX(P(ligne_valeurs[-1]))
            #===================================================================
            # var_de_P += "\\txb{\\rnode{neu%s}{%s}}\\cr" % (compteur, TeX(P(ligne_valeurs[-1])))
            #===================================================================

        cor.append(var_de_P)
        #=======================================================================
        # cor.append("$$ \\tabvar{\n %s\n %s\n %s}$$" % \
        #            (str_valeurs, str_signes, var_de_P))
        #=======================================================================
        #=======================================================================
        # for i in range(1, compteur + 1):
        #     cor.append("\\ncline[nodesep=0.15,linewidth=0.5pt]{->}{neu%s}{neu%s}" % (i - 1, i))
        #=======================================================================
        cor.append(r'\end{tikzpicture}\par')

        if borneinf == float("-inf"):
            cor.append(u"$$\\lim_{%s\\to %s} %s= \\lim_{%s\\to %s} %s%s^3=%s $$  " % (var, "-\\infty", P, var, "-\\infty", P[3], var, TeX(P(float("-inf")))))
        if bornesup == float("+inf"):
            cor.append(u"$$\\lim_{%s\\to %s} %s= \\lim_{%s\\to %s} %s%s^3=%s  $$ " % (var, "+\\infty", P, var, "+\\infty", P[3], var, TeX(P(float("inf")))))
    return exo, cor


                ###############################################
                #                                             #
                #   Fonctions étudiant les polynômes          #
                #                                             #
                ###############################################

def tableau_de_signe(P, nomP, delta, racines, cor, borneinf=float("-inf"), bornesup=float("+inf"), detail=False):
    '''Étudie le signe d'un polynôme de degré2'''
    ''' ne fonctionne pas si on a pas borneinf < x1< x2 <bornesup'''
    '''detail=True permet de récupérer la dernière ligne avec le signe de P'''
    var = P.var
    signe_moinsa, signe_a = [('+', '-'), ('-', '+')][P[2] > 0]  # Ca donne bien ce qu'on veut...
    signes = []
    str_variables = "{$%s$/1, $%s\\,(x)$/1}" % (var, nomP)
    if delta < 0:
        cor.append(_("Comme $\\Delta <0$, $%s(%s)$ ne s'annule pas et est toujours du signe de $a$") % (nomP, var))
        cor.append(_("Ainsi "))
        str_valeurs = "{$%s$,$%s$}" % (TeX(borneinf), TeX(bornesup))
        str_signes = "\\tkzTabLine{,%s}" % signe_a
        # str_valeurs = "\\tx{%s}&\\tx{%s}&& \\tx{%s}\\cr" % (var, TeX(borneinf), TeX(bornesup))
        # str_signe = "\\tx{%s(%s)}&&\\tx{%s}&\\cr" % (nomP, var, signe_a)
        signes = [signe_a]
        ligne_valeurs = [borneinf, bornesup]
    elif delta == 0:
        cor.append(_("Comme $\\Delta =0$, $%s(%s)$ s'annule une seule fois pour $%s_0=%s$ et est toujours du signe de $a$.\\par") % (nomP, var, var, racines[0]))
        if racines[0] < borneinf or racines[0] > bornesup:
            if borneinf != float("-inf"):
                intervalle = "["
            else:
                intervalle = "]"
            intervalle += "%s~;~%s" % (TeX(borneinf), TeX(bornesup))
            if bornesup == float("inf"):
                intervalle += "["
            else:
                intervalle += "]"
            cor.append(_("Or $%s$ n'est pas dans l'intervalle $%s$ donc ") % (TeX(racines[0]), intervalle))
            ligne_valeurs = [borneinf, bornesup]

            str_valeurs = "{$%s$,$%s$}" % (TeX(borneinf), TeX(bornesup))
            str_signes = "\\tkzTabLine{,%s}" % signe_a
#===============================================================================
#             str_valeurs = "\\tx{%s}&\\tx{%s}&& \\tx{%s}\\cr" % (var, TeX(borneinf), TeX(bornesup))
#
#             str_signe = "\\tx{%s(%s)}&&\\tx{%s}&\\cr" % (nomP, var, signe_a)
#===============================================================================
            signes = [signe_a]
        else:

            ligne_valeurs = [borneinf, racines[0], bornesup]
            str_valeurs = "{$%s$,$%s$,$%s$}" % (TeX(borneinf), TeX(racines[0]), TeX(bornesup))
            str_signes = "\\tkzTabLine{,%s,0,%s}" % (signe_a, signe_a)
            #===================================================================
            # str_valeurs = "\\tx{%s}&\\tx{%s}&& \\tx{%s}&& \\tx{%s}\\cr" % (var, TeX(borneinf), TeX(racines[0]), TeX(bornesup))
            # str_signe = "\\tx{%s(%s)}&&\\tx{%s}&\\tx{0}&\\tx{%s}&\\cr" % (nomP, var, signe_a, signe_a)
            #===================================================================
            signes = [signe_a, signe_a]
    elif delta > 0:
        [x1, x2] = racines
        cor.append(_("Comme $\\Delta >0$, $%s(%s)$ est du signe de $-a$ entre les racines.") % (nomP, var))
        compare = [(x1 <= borneinf) + (x2 <= borneinf), (x2 >= bornesup) + (x1 >= bornesup)]
        # x_x1=x_x2=sign_x1=sign_x2=""
        if compare != [0, 0]:
            if borneinf != float("-inf"):
                intervalle = "["
            else:
                intervalle = "]"
            intervalle += "%s~;~%s" % (TeX(borneinf), TeX(bornesup))
            if bornesup == float("inf"):
                intervalle += "["
            else:
                intervalle += "]"

        ligne_valeurs = [borneinf]
        if compare[0] >= 1 or compare[1] == 2:
            x_x1 = sign_x1 = ""
        else:
            x_x1 = "$%s$," % (TeX(x1))
            ligne_valeurs += [x1]
            sign_x1 = ",%s,0" % (signe_a)
            signes += [signe_a]

        if 2 in compare:
            entreracines = ",%s" % (signe_a)
            signes += [signe_a]
        else:
            entreracines = ",%s" % (signe_moinsa)
            signes += [signe_moinsa]

        if compare[1] >= 1 or compare[0] == 2:
            x_x2 = sign_x2 = ""
        else:
            x_x2 = "$%s$," % (TeX(x2))
            ligne_valeurs += [x2]
            sign_x2 = ",0,%s" % (signe_a)
            signes += [signe_a]
        ligne_valeurs += [bornesup]
        # Ne rien dire si une racine est égale à une borne
        compare = [compare[0] - (x1 == borneinf) - (x2 == borneinf), compare[1] - (x1 == bornesup) - (x2 == bornesup)]

        if sum(compare) == 2 :
            # les deux racines sont à supprimer
            cor.append(_("\\par Or $%s$ et $%s$ ne sont pas dans $%s$.") % (TeX(x1), TeX(x2), intervalle))

        elif compare[0] == 1:
            cor.append(_("\\par Or $%s$ n'est pas dans $%s$.") % (TeX(x1), intervalle))
        elif compare[1] == 1:
            cor.append(_("\\par Or $%s$ n'est pas dans $%s$.") % (TeX(x2), intervalle))
        cor.append("Ainsi \\par")

        str_valeurs = "{$%s$, %s %s $%s$}" % (TeX(borneinf), x_x1, x_x2, TeX(bornesup))
        str_signes = "\\tkzTabLine{%s%s%s}" % (sign_x1, entreracines, sign_x2)

#===============================================================================
#         str_valeurs = "\\tx{%s}& \\tx{%s}& %s %s & \\tx{%s}\\cr" % (var, TeX(borneinf), x_x1, x_x2, TeX(bornesup))
#
#         str_signe = "\\tx{%s(%s)}&%s %s %s&\\cr" % (nomP, var, sign_x1, entreracines, sign_x2)
#===============================================================================

    cor.append("\\begin{tikzpicture}\n\\tkzTabInit[espcl=2.5]")
    cor.append(str_variables)
    cor.append(str_valeurs)
    cor.append(str_signes)
    cor.append(r'\end{tikzpicture}\par')
    #===========================================================================
    # cor.append("$$\\tabvar{")
    # cor.append(str_valeurs)
    # cor.append("%s}$$" % (str_signe))
    #===========================================================================
    if detail:
        return str_variables, str_signes, str_valeurs, signes, ligne_valeurs

def factorise_identites_remarquables(pol1, sgns, var='', racines=True):
    '''Factorise un polynôme grâce aux identités remarquables'''
    from pyromaths.classes.Fractions import Fraction
    if var == '':
        var = pol1.var
    X = Polynome({1:1}, var)
    a1 = pgcd(int(pol1[0]), pgcd(int(pol1[1]), int(pol1[2])))  # du signe de a=pol1[2]
    if a1 != 1 or a1 != -1:
        pol2 = pol1 / a1
    else:
        pol2 = pol1
    # coeff=coeff/int(math.sqrt(a1))
    # pol2=(cx)^2 ±2× cx × b + b^2
    # pol2[2]=c^2
    # pol2[1]=2× cx × b
    # pol2[0] = b^2
    c = int(sqrt(pol2[2]))  # problème d'arrondi ?

    factorisation = []
    if a1 != 1:
        pol_temp = (pol1 / a1).simplifie()
        pol_temp.var = var
        factorisation.append("%s \\times{}\\big[ %s \\big]" % (TeX(a1), pol_temp))
        facteur2 = "%s\\times{} \\big[" % (TeX(a1))
    else:
        facteur2 = ""
    if c != 1:
        facteur2 += u"(%s %s)^2" % (TeX(c), var)
    else:
        facteur2 += u"%s^2" % var
    if sgns:
        if sgns == -2:  # -2 => (a-b)²
            facteur2 += "-2 \\times{} "
        else:  # +2 => (a+b)²
            facteur2 += "+2 \\times{} "
        if c == 1:
            facteur2 += var
        else:
            facteur2 += TeX(c) + var
        b = int(sqrt(pol2[0]))
        facteur2 += " \\times{} %s +" % (TeX(b))
    else:
        # a²-b²
        facteur2 += "-"
        b = int(sqrt(-(pol2[0])))
    facteur2 += u"%s^2" % (TeX(b))
    if a1 != 1:
        facteur2 += "\\big]"
    factorisation.append(facteur2)
    facteur3 = ""
    if a1 != 1:
        facteur3 += TeX(a1)
    sgns = sgns //  2
    if sgns:  # (cx-b)² ou (cx+b)²
        liste_racines = [Fraction(-(sgns)) * b / c]
        facteur3 += "{(%s)}^2" % (c * X + sgns * b)
    else:  # (cx-b)(cx+b)
        liste_racines = [Fraction(-1) * b / c, Fraction(1) * b / c]
        facteur3 += "(%s)(%s)" % (c * X + b, c * X - b)
    factorisation.append(facteur3)
    if racines:
        return factorisation, liste_racines
    return factorisation

def racines_degre2(P):
    """renvoie les racines d'un polynôme de degré 2"""
    from pyromaths.classes.Fractions import Fraction
    delta = int(P[1] ** 2 - 4 * P[2] * P[0])
    if delta == 0:
        x0 = eval(priorites('Fraction(-1, 2)*%r/%r' % (P[1], P[2]))[-1][0])
        if isinstance(x0, (Fraction, RacineDegre2)):
            liste_racines = [x0.simplifie()]
        else:
            liste_racines = [x0]
        liste_str_racines = ["\\dfrac{-%s}{2\\times{} %s}" % (pTeX(P[1]), pTeX(P[2]))]
        simplrac = [False]
    elif delta > 0:
        simplrac, strx1, x1, strx2, x2 = listeracines(P[2], P[1], delta, parentheses=False)
        liste_racines = [x1, x2]
        liste_str_racines = [strx1, strx2]
    else:
        simplrac = [False]
        liste_racines = liste_str_racines = []
    return delta, simplrac, liste_racines, liste_str_racines
    # delta
    # simplrac[0] est True si racine de Delta se simplifie, alors simplrac[1] est la racine de delta simplifiée
    # liste_racines donne la liste des racines sous la forme simplifiée, sous forme numérique si possible
    # liste_str_racine est la liste des racines au format TeX, non simplifié.

def listeracines(a, b, delta, parentheses=False):
    '''renvoie racsimple,simplifie,formule_x1,x_1,formule_x2,x2'''
    '''avec x_1<x_2
       si parenthese=True, renvoie deux booleens
          parenthesex1=True signifie qu'il faut mettre des parenthese autour de x1
       On suppose delta >0
       simplrac est True si racine de delta se simplifie'''
    a = int(a)
    b = int(b)
#     parenthesex1 = parenthesex2 = True  # par défaut
    # simplrac=True
    strx1 = "\\dfrac{-%s-\\sqrt{%s}}{2\\times{} %s}" % (pTeX(b), TeX(delta), pTeX(a))
    strx2 = "\\dfrac{-%s+\\sqrt{%s}}{2\\times{} %s}" % (pTeX(b), TeX(delta), pTeX(a))
    # #on a strx1<strx2
    coeff, radicande = simplifie_racine(delta)

    # x1,x2 simplifiés ont une écriture fractionnaire donc
    parenthesex1 = parenthesex2 = False
    if coeff == 1:  # delta n'a pas de facteur carré, on ne peut rien simplifier
        rac_delta = radicalTeX(delta)
        simplrac = [False]
    else:
        if radicande == 1:
            rac_delta = TeX(coeff)
        else:
            rac_delta = TeX(coeff) + radicalTeX(radicande)
        simplrac = [True, rac_delta]
    x1 = RacineDegre2(-b, 2 * a, -1, delta)
    x2 = RacineDegre2(-b, 2 * a, 1, delta)
    if b == 0:
        parenthesex1 = (coeff * a > 0)
        parenthesex2 = (coeff * a < 0)
    if a < 0:
        strx1, strx2, x1, x2, parenthesex1, parenthesex2 = strx2, strx1, x2, x1, parenthesex2, parenthesex1
    if parentheses:
        return simplrac, strx1, x1, strx2, x2, parenthesex1, parenthesex2
    else:
        return simplrac, strx1, x1, strx2, x2
        # simplrac[0] est True si racine de Delta se simplifie, alors simplrac[1] est la racine de delta simplifiée

def factorisation_degre2(P, factorisation=True):
    # x1=x2=0
    from pyromaths.classes.Fractions import Fraction

    var = P.var
    X = Polynome({1:1}, var)
    delta = int(eval(priorites('%r**2-4*%r*%r' % (P[1], P[2], P[0]))[-1][0]))
    if delta < 0:
        factorisation = []
        str_racines = []
        racines = []
        simplrac = [False]
    elif delta == 0:
        x0 = eval(priorites('Fraction(-1, 2)*%r/%r' % (P[1], P[2]))[-1][0])
        simplrac = [False]
        if isinstance(x0, (Fraction, RacineDegre2)):
            racines = [x0.simplifie()]
        else:
            racines = [x0]
        str_racines = ["\\dfrac{-%s}{2\\times{} %s}" % (pTeX(P[1]), pTeX(P[2]))]


        P0 = "%s-%s" % (var, pTeX(racines[0]))
        factorisation = [[P0, P0]]  # non simplifiée

        if 0 > racines[0]:
            P0 = X - racines[0]
            factorisation.append([P0, P0])
    else:  # delta>0
        simplrac, strx1, x1, strx2, x2 = listeracines(P[2], P[1], delta)
        if isinstance(x1, RacineDegre2):
            x1 = x1.simplifie()
            x2 = x2.simplifie()
        racines = [x1, x2]
        str_racines = [strx1, strx2]
        P1 = "%s-%s" % (var, pTeX(x1))
        P2 = "%s-%s" % (var, pTeX(x2))
        factorisation = [[P1, P2]]  # non simplifiée
        # Peut-on simplifier les parenthèses ?
        if x1.radicande == 0 and (x1.numerateur < 0 or x2.numerateur < 0):
            P1 = (X - x1)(var)
            P2 = (X - x2)(var)
            factorisation.append([P1, P2])
    return delta, simplrac, racines, str_racines, factorisation


def factorisation_degre3(E, nomE, exo=[], cor=[], racines=[0, 1, -1, 2, -2]):
    '''Factorise un polynôme de degré 3 avec une racine évidente'''
    var = E.var
    X = Polynome({1:1}, var)
    exo.append(_("\\item Soit $%s =%s $") % (nomE, E))
    cor.append(_("\\item Soit $%s=%s $)") % (nomE, E))
    exo.append("\\begin{enumerate}")
    cor.append("\\begin{enumerate}")
    for x0 in racines:
        if E(x0) == 0:
            break
    if x0 in [-2, -1, 0, 1, 2]:
        exo.append(_(u"\\item Vérifier si $%s $ possède une racine évidente.") % (nomE))
    else:
        exo.append(_(u"\\item Vérifier que $%s$ est une racine de $%s$.") % (TeX(x0), nomE))
    exo.append(_("\\item Factoriser $%s $.") % (nomE))
    cor.append("\\item ")

    if x0 == 0:
        degre_facteur = min(E.puiss)
        # degre_facteur=1
        E2 = (E / (X ** degre_facteur))[0]
        if degre_facteur == 1:
            cor.append(_("On remarque que $%s$ peut se factoriser par $%s$ et $%s=%s\\left(%s\\right)$") % (nomE, var, nomE, var, E2))
        elif degre_facteur == 2:
            cor.append(_(u"On remarque que $%s$ peut se factoriser par $%s^2$ et $%s=%s^2\\left(%s\\right)$")\
            % (nomE, E.var, nomE, E.var, E2))
            exo.append("\\end{enumerate}")
            cor.append("\\end{enumerate}")
            return exo, cor
    else:
        cor.append(_("Comme $%s(%s)=0$, on peut diviser $%s$ par $%s$") % (nomE, TeX(x0), nomE, X - x0))
        cor.append(TeX_division(E, (X - x0)) + "")
        E2, reste = E / (X - x0)
    cor.append(_("\\item On doit maintenant factoriser le polynôme $%s_2=%s$\\\\") % (nomE, E2))
    delta, simplrac, racines, str_racines, factorisation = factorisation_degre2(E2, factorisation=True)
    cor = redaction_factorisation(E2, nomP=nomE + "_2", exo=[], cor=cor)[1]
    cor.append("\\par")
    cor.append(_("On en conclue donc que $%s=") % (nomE))
#     final = 0
    if x0 == 0:
        P0 = E.var
    else:
        P0 = "\\left(%s\\right)" % (X - x0)
    if E[3] == -1:
        cor.append("-")
    elif E[3] != 1:
        cor.append(TeX(E[3]))
    if delta < 0:
        # P1 = factorisation[-1][0]
        E_factorise = "%s\\times{}%s$" % (P0, E2)
    elif delta == 0:
        P1 = factorisation[-1][0]
        E_factorise = "%s\\times{}{\\left(%s\\right)}^2$" % (P0, P1)
    else:
        P1 = factorisation[-1][0]
        P2 = factorisation[-1][1]
        E_factorise = "%s\\left(%s\\right)\\left(%s\\right)$" % (P0, P1, P2)
    cor.append(E_factorise)

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo, cor


#----------------- redaction ---------------------------------------------------------

def redaction_factorisation(P, nomP="P", exo=[], cor=[]):
    var = P.var
    delta, simpl_delta, racines, str_racines, factorisation = factorisation_degre2(P)
    redaction_racines(P, nomP, var, cor)

    # factorisation
    if delta < 0:
        cor.append(_(u"On ne peut pas factoriser $%s(%s)$.") % (nomP, var))
    elif delta == 0:
        cor.append(_(u"On peut donc écrire "))
        ligne_factorisation = "$$%s(%s)" % (nomP, var)
        for etape in factorisation:
            ligne_factorisation += " = "
            if P[2] != 1:
                ligne_factorisation += "%s \\times{} " % (TeX(P[2]))

            ligne_factorisation += "{\\left(%s\\right)}^2" % (etape[0])
        ligne_factorisation += "$$"
        cor.append(ligne_factorisation)
    else:
        cor.append(_(u"On peut donc écrire "))
        ligne_factorisation = "$$%s(%s)" % (nomP, var)
        for etape in factorisation:
            ligne_factorisation += " = "
            if P[2] != 1:
                ligne_factorisation += "%s \\times{} " % (TeX(P[2]))
            if len(etape) == 1:
                ligne_factorisation += etape[0]
            else:
                ligne_factorisation += "\\left(%s\\right)\left(%s\\right)" % (etape[0], etape[1])
        ligne_factorisation += "$$"
        cor.append(ligne_factorisation)
    return exo, cor

def redaction_racines(P, nomP, var, cor=[]):
    delta, simpl_delta, liste_racines, liste_str_racines = racines_degre2(P)
    ligne_delta = _(u"Je calcule $\\Delta=%s^2-4\\times{} %s\\times{} %s=%s$") % (pTeX(P[1]), pTeX(P[2]), pTeX(P[0]), TeX(delta))
    if simpl_delta[0]:
        ligne_delta += _(" et $%s=%s$.\\par") % (radicalTeX(delta), simpl_delta[1])
    else:
        ligne_delta += ".\\par"
    cor.append(ligne_delta)
    if delta < 0:
        cor.append(_("Comme $\\Delta <0$, $%s(%s)$ n'a pas de racines.") % (nomP, var))
    elif delta == 0:
        cor.append(_("Comme $\\Delta=0$, $%s(%s)$ a une seule racine $%s_0=%s=%s$.\\par") % (nomP, var, var, liste_str_racines[0], TeX(liste_racines[0])))
    else:  # delta>0
        [x1, x2] = liste_racines
        cor.append(_("Comme $\\Delta>0$, $%s(%s)$ a deux racines :") % (nomP, var))
        if isinstance(x1, RacineDegre2):
#             simplification1 = simplification2 = ""
            x1, detail1 = x1.simplifie(True)
            x2, detail2 = x2.simplifie(True)
            max_len = max(len(detail1), len(detail2))
            cor.append("\\begin{align*}")
            cor.append("%s &= %s  &%s &= %s" % \
                       (liste_str_racines[0], liste_racines[0], liste_str_racines[1], liste_racines[1]))
            cor.append("\\\\")
            for i in range(0, max_len):
                if i < len(detail1):
                    cor.append("&= %s&" % (detail1[i]))
                else:
                    cor.append("&&")
                if i < len(detail2):
                    cor.append("&= %s" % (detail2[i]))
                else:
                    cor.append("& ")
                cor.append("\\\\")
            cor.pop(-1)
            cor.append("\\end{align*}")
            cor.append(_("Les racines de $%s$ sont $%s_1=%s$ et $%s_2=%s$.\\par") % (nomP, var, x1, var, x2))
        else:
            [strx1, strx2] = liste_str_racines
            cor.append(_("Les racines de $%s$ sont $%s_1=%s=%s$ et $%s_2=%s=%s$.") % (nomP, var, strx1, x1, var, strx2, x2))

    return cor



#!/usr/bin/env python3

# Copyright (C) 2018 -- Louis Paternault (spalax@gresille.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

"""Création d'un modèle d'exercice."""

import logging
import os
import shutil
import textwrap

from pyromaths.ex import TexExercise
from pyromaths.outils.System import Fiche

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s %(message)s',
    )

class DummyExercise(TexExercise):
    """Faux exercice, servant de modèle pour les nouveaux exercices."""

    tags = []

    def tex_statement(self):
        return textwrap.dedent(r"""
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % DÉBUT DE L'ÉNONCÉ
            \exercice

            ÉNONCÉ DE L'EXERCICE

            % FIN DE L'ÉNONCÉ
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        """)

    def tex_answer(self):
        return textwrap.dedent(r"""
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % DÉBUT DU CORRIGÉ
            \exercice*

            CORRIGÉ DE L'EXERCICE

            % FIN DU CORRIGÉ
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        """)

PARAMETRES = {
    'enonce': True,
    'corrige': True,
    'exercices': [DummyExercise()],
    }
TEXFILE = "dummy.tex"
PDFFILE = "dummy.pdf"

def main():
    """Fonction principale."""
    with Fiche(PARAMETRES) as fiche:
        fiche.write_tex()
        shutil.copy(fiche.texname, TEXFILE)
        fiche.write_pdf()
        shutil.copy(fiche.pdfname, PDFFILE)
        fiche.show_pdf(PDFFILE)
    logging.info("Le modèle de document est disponible dans le fichier '%s'.", TEXFILE)

if __name__ == "__main__":
    main()
